# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=R0902,protected-access

from typing import Any, List, Optional, Union

from azure.ai.ml._restclient.v2023_04_01_preview.models import ClassificationModels
from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    ClassificationTrainingSettings as RestClassificationTrainingSettings,
)
from azure.ai.ml._restclient.v2023_04_01_preview.models import ForecastingModels
from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    ForecastingTrainingSettings as RestForecastingTrainingSettings,
)
from azure.ai.ml._restclient.v2023_04_01_preview.models import RegressionModels
from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    RegressionTrainingSettings as RestRegressionTrainingSettings,
)
from azure.ai.ml._restclient.v2023_04_01_preview.models import TrainingSettings as RestTrainingSettings
from azure.ai.ml._utils.utils import camel_to_snake, from_iso_duration_format_mins, to_iso_duration_format_mins
from azure.ai.ml.constants import TabularTrainingMode
from azure.ai.ml.entities._job.automl.stack_ensemble_settings import StackEnsembleSettings
from azure.ai.ml.entities._mixins import RestTranslatableMixin
from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationException


class TrainingSettings(RestTranslatableMixin):
    """TrainingSettings class for Azure Machine Learning."""

    def __init__(
        self,
        *,
        enable_onnx_compatible_models: Optional[bool] = None,
        enable_dnn_training: Optional[bool] = None,
        enable_model_explainability: Optional[bool] = None,
        enable_stack_ensemble: Optional[bool] = None,
        enable_vote_ensemble: Optional[bool] = None,
        stack_ensemble_settings: Optional[StackEnsembleSettings] = None,
        ensemble_model_download_timeout: Optional[int] = None,
        allowed_training_algorithms: Optional[List[str]] = None,
        blocked_training_algorithms: Optional[List[str]] = None,
        training_mode: Optional[Union[str, TabularTrainingMode]] = None,
    ):
        """TrainingSettings class for Azure Machine Learning.

        :param enable_onnx_compatible_models: If set to True, the model will be trained to be compatible with ONNX
        :type enable_onnx_compatible_models: typing.Optional[bool]
        :param enable_dnn_training: If set to True,the model will use DNN training
        :type enable_dnn_training: typing.Optional[bool]
        :param enable_model_explainability: If set to True, the model will be trained to be explainable
        :type enable_model_explainability: typing.Optional[bool]
        :param enable_stack_ensemble: If set to True, a final ensemble model will be created using a stack of models
        :type enable_stack_ensemble: typing.Optional[bool]
        :param enable_vote_ensemble: If set to True, a final ensemble model will be created using a voting ensemble
        :type enable_vote_ensemble: typing.Optional[bool]
        :param stack_ensemble_settings: Settings for stack ensemble
        :type stack_ensemble_settings: typing.Optional[azure.ai.ml.automl.StackEnsembleSettings]
        :param ensemble_model_download_timeout: Timeout for downloading ensemble models
        :type ensemble_model_download_timeout: typing.Optional[typing.List[int]]
        :param allowed_training_algorithms: Models to train
        :type allowed_training_algorithms: typing.Optional[typing.List[str]]
        :param blocked_training_algorithms: Models that will not be considered for training
        :type blocked_training_algorithms: typing.Optional[typing.List[str]]
        :param training_mode: [Experimental] The training mode to use.
            The possible values are-

            * distributed- enables distributed training for supported algorithms.

            * non_distributed- disables distributed training.

            * auto- Currently, it is same as non_distributed. In future, this might change.

            Note: This parameter is in public preview and may change in future.
        :type training_mode: typing.Optional[typing.Union[str, azure.ai.ml.constants.TabularTrainingMode]]
        """
        self.enable_onnx_compatible_models = enable_onnx_compatible_models
        self.enable_dnn_training = enable_dnn_training
        self.enable_model_explainability = enable_model_explainability
        self.enable_stack_ensemble = enable_stack_ensemble
        self.enable_vote_ensemble = enable_vote_ensemble
        self.stack_ensemble_settings = stack_ensemble_settings
        self.ensemble_model_download_timeout = ensemble_model_download_timeout
        self.allowed_training_algorithms = allowed_training_algorithms
        self.blocked_training_algorithms = blocked_training_algorithms
        self.training_mode = training_mode

    @property
    def training_mode(self) -> Optional[TabularTrainingMode]:
        return self._training_mode

    @training_mode.setter
    def training_mode(self, value: Optional[Union[str, TabularTrainingMode]]) -> None:
        if value is None or value is TabularTrainingMode:
            self._training_mode = value
        elif hasattr(TabularTrainingMode, camel_to_snake(value).upper()):
            self._training_mode = TabularTrainingMode[camel_to_snake(value).upper()]
        else:
            supported_values = ", ".join([f'"{camel_to_snake(mode.value)}"' for mode in TabularTrainingMode])
            msg = (
                f"Unsupported training mode: {value}. Supported values are- {supported_values}. "
                "Or you can use azure.ai.ml.constants.TabularTrainingMode enum."
            )
            raise ValidationException(
                message=msg,
                no_personal_data_message=msg,
                target=ErrorTarget.AUTOML,
                error_category=ErrorCategory.USER_ERROR,
            )

    @property
    def allowed_training_algorithms(self) -> Optional[List[str]]:
        return self._allowed_training_algorithms

    @allowed_training_algorithms.setter
    def allowed_training_algorithms(self, value: Optional[List[str]]) -> None:
        self._allowed_training_algorithms = value

    @property
    def blocked_training_algorithms(self) -> Optional[List[str]]:
        return self._blocked_training_algorithms

    @blocked_training_algorithms.setter
    def blocked_training_algorithms(self, value: Optional[List[str]]) -> None:
        self._blocked_training_algorithms = value

    def _to_rest_object(self) -> RestTrainingSettings:
        return RestTrainingSettings(
            enable_dnn_training=self.enable_dnn_training,
            enable_onnx_compatible_models=self.enable_onnx_compatible_models,
            enable_model_explainability=self.enable_model_explainability,
            enable_stack_ensemble=self.enable_stack_ensemble,
            enable_vote_ensemble=self.enable_vote_ensemble,
            stack_ensemble_settings=(
                self.stack_ensemble_settings._to_rest_object() if self.stack_ensemble_settings else None
            ),
            ensemble_model_download_timeout=to_iso_duration_format_mins(self.ensemble_model_download_timeout),
            training_mode=self.training_mode,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestTrainingSettings) -> "TrainingSettings":
        return cls(
            enable_dnn_training=obj.enable_dnn_training,
            enable_onnx_compatible_models=obj.enable_onnx_compatible_models,
            enable_model_explainability=obj.enable_model_explainability,
            enable_stack_ensemble=obj.enable_stack_ensemble,
            enable_vote_ensemble=obj.enable_vote_ensemble,
            ensemble_model_download_timeout=from_iso_duration_format_mins(obj.ensemble_model_download_timeout),
            stack_ensemble_settings=(
                StackEnsembleSettings._from_rest_object(obj.stack_ensemble_settings)
                if obj.stack_ensemble_settings
                else None
            ),
            training_mode=obj.training_mode,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, TrainingSettings):
            return NotImplemented
        return (
            self.enable_dnn_training == other.enable_dnn_training
            and self.enable_onnx_compatible_models == other.enable_onnx_compatible_models
            and self.enable_model_explainability == other.enable_model_explainability
            and self.enable_stack_ensemble == other.enable_stack_ensemble
            and self.enable_vote_ensemble == other.enable_vote_ensemble
            and self.ensemble_model_download_timeout == other.ensemble_model_download_timeout
            and self.stack_ensemble_settings == other.stack_ensemble_settings
            and self.allowed_training_algorithms == other.allowed_training_algorithms
            and self.blocked_training_algorithms == other.blocked_training_algorithms
            and self.training_mode == other.training_mode
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)


class ClassificationTrainingSettings(TrainingSettings):
    """Classification TrainingSettings class for Azure Machine Learning."""

    def __init__(
        self,
        **kwargs: Any,
    ):
        super().__init__(**kwargs)

    @property
    def allowed_training_algorithms(self) -> Optional[List]:
        return self._allowed_training_algorithms

    @allowed_training_algorithms.setter
    def allowed_training_algorithms(self, allowed_model_list: Union[List[str], List[ClassificationModels]]) -> None:
        self._allowed_training_algorithms = (
            None
            if allowed_model_list is None
            else [ClassificationModels[camel_to_snake(o)] for o in allowed_model_list]
        )

    @property
    def blocked_training_algorithms(self) -> Optional[List]:
        return self._blocked_training_algorithms

    @blocked_training_algorithms.setter
    def blocked_training_algorithms(self, blocked_model_list: Union[List[str], List[ClassificationModels]]) -> None:
        self._blocked_training_algorithms = (
            None
            if blocked_model_list is None
            else [ClassificationModels[camel_to_snake(o)] for o in blocked_model_list]
        )

    def _to_rest_object(self) -> RestClassificationTrainingSettings:
        return RestClassificationTrainingSettings(
            enable_dnn_training=self.enable_dnn_training,
            enable_onnx_compatible_models=self.enable_onnx_compatible_models,
            enable_model_explainability=self.enable_model_explainability,
            enable_stack_ensemble=self.enable_stack_ensemble,
            enable_vote_ensemble=self.enable_vote_ensemble,
            stack_ensemble_settings=self.stack_ensemble_settings,
            ensemble_model_download_timeout=to_iso_duration_format_mins(self.ensemble_model_download_timeout),
            allowed_training_algorithms=self.allowed_training_algorithms,
            blocked_training_algorithms=self.blocked_training_algorithms,
            training_mode=self.training_mode,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestClassificationTrainingSettings) -> "ClassificationTrainingSettings":
        return cls(
            enable_dnn_training=obj.enable_dnn_training,
            enable_onnx_compatible_models=obj.enable_onnx_compatible_models,
            enable_model_explainability=obj.enable_model_explainability,
            enable_stack_ensemble=obj.enable_stack_ensemble,
            enable_vote_ensemble=obj.enable_vote_ensemble,
            ensemble_model_download_timeout=from_iso_duration_format_mins(obj.ensemble_model_download_timeout),
            stack_ensemble_settings=obj.stack_ensemble_settings,
            allowed_training_algorithms=obj.allowed_training_algorithms,
            blocked_training_algorithms=obj.blocked_training_algorithms,
            training_mode=obj.training_mode,
        )


class ForecastingTrainingSettings(TrainingSettings):
    """Forecasting TrainingSettings class for Azure Machine Learning."""

    def __init__(
        self,
        **kwargs: Any,
    ):
        super().__init__(**kwargs)

    @property
    def allowed_training_algorithms(self) -> Optional[List]:
        return self._allowed_training_algorithms

    @allowed_training_algorithms.setter
    def allowed_training_algorithms(self, allowed_model_list: Union[List[str], List[ForecastingModels]]) -> None:
        self._allowed_training_algorithms = (
            None if allowed_model_list is None else [ForecastingModels[camel_to_snake(o)] for o in allowed_model_list]
        )

    @property
    def blocked_training_algorithms(self) -> Optional[List]:
        return self._blocked_training_algorithms

    @blocked_training_algorithms.setter
    def blocked_training_algorithms(self, blocked_model_list: Union[List[str], List[ForecastingModels]]) -> None:
        self._blocked_training_algorithms = (
            None if blocked_model_list is None else [ForecastingModels[camel_to_snake(o)] for o in blocked_model_list]
        )

    def _to_rest_object(self) -> RestForecastingTrainingSettings:
        return RestForecastingTrainingSettings(
            enable_dnn_training=self.enable_dnn_training,
            enable_onnx_compatible_models=self.enable_onnx_compatible_models,
            enable_model_explainability=self.enable_model_explainability,
            enable_stack_ensemble=self.enable_stack_ensemble,
            enable_vote_ensemble=self.enable_vote_ensemble,
            stack_ensemble_settings=self.stack_ensemble_settings,
            ensemble_model_download_timeout=to_iso_duration_format_mins(self.ensemble_model_download_timeout),
            allowed_training_algorithms=self.allowed_training_algorithms,
            blocked_training_algorithms=self.blocked_training_algorithms,
            training_mode=self.training_mode,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestForecastingTrainingSettings) -> "ForecastingTrainingSettings":
        return cls(
            enable_dnn_training=obj.enable_dnn_training,
            enable_onnx_compatible_models=obj.enable_onnx_compatible_models,
            enable_model_explainability=obj.enable_model_explainability,
            enable_stack_ensemble=obj.enable_stack_ensemble,
            enable_vote_ensemble=obj.enable_vote_ensemble,
            ensemble_model_download_timeout=from_iso_duration_format_mins(obj.ensemble_model_download_timeout),
            stack_ensemble_settings=obj.stack_ensemble_settings,
            allowed_training_algorithms=obj.allowed_training_algorithms,
            blocked_training_algorithms=obj.blocked_training_algorithms,
            training_mode=obj.training_mode,
        )


class RegressionTrainingSettings(TrainingSettings):
    """Regression TrainingSettings class for Azure Machine Learning."""

    def __init__(
        self,
        **kwargs: Any,
    ):
        super().__init__(**kwargs)

    @property
    def allowed_training_algorithms(self) -> Optional[List]:
        return self._allowed_training_algorithms

    @allowed_training_algorithms.setter
    def allowed_training_algorithms(self, allowed_model_list: Union[List[str], List[ForecastingModels]]) -> None:
        self._allowed_training_algorithms = (
            None if allowed_model_list is None else [RegressionModels[camel_to_snake(o)] for o in allowed_model_list]
        )

    @property
    def blocked_training_algorithms(self) -> Optional[List]:
        return self._blocked_training_algorithms

    @blocked_training_algorithms.setter
    def blocked_training_algorithms(self, blocked_model_list: Union[List[str], List[ForecastingModels]]) -> None:
        self._blocked_training_algorithms = (
            None if blocked_model_list is None else [RegressionModels[camel_to_snake(o)] for o in blocked_model_list]
        )

    def _to_rest_object(self) -> RestRegressionTrainingSettings:
        return RestRegressionTrainingSettings(
            enable_dnn_training=self.enable_dnn_training,
            enable_onnx_compatible_models=self.enable_onnx_compatible_models,
            enable_model_explainability=self.enable_model_explainability,
            enable_stack_ensemble=self.enable_stack_ensemble,
            enable_vote_ensemble=self.enable_vote_ensemble,
            stack_ensemble_settings=self.stack_ensemble_settings,
            ensemble_model_download_timeout=to_iso_duration_format_mins(self.ensemble_model_download_timeout),
            allowed_training_algorithms=self.allowed_training_algorithms,
            blocked_training_algorithms=self.blocked_training_algorithms,
            training_mode=self.training_mode,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestRegressionTrainingSettings) -> "RegressionTrainingSettings":
        return cls(
            enable_dnn_training=obj.enable_dnn_training,
            enable_onnx_compatible_models=obj.enable_onnx_compatible_models,
            enable_model_explainability=obj.enable_model_explainability,
            enable_stack_ensemble=obj.enable_stack_ensemble,
            enable_vote_ensemble=obj.enable_vote_ensemble,
            ensemble_model_download_timeout=from_iso_duration_format_mins(obj.ensemble_model_download_timeout),
            stack_ensemble_settings=obj.stack_ensemble_settings,
            allowed_training_algorithms=obj.allowed_training_algorithms,
            blocked_training_algorithms=obj.blocked_training_algorithms,
            training_mode=obj.training_mode,
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

from marshmallow import fields, post_load, validate

from azure.ai.ml._schema.core.schema_meta import PatchedSchemaMeta


class CommandJobLimitsSchema(metaclass=PatchedSchemaMeta):
    timeout = fields.Int()

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities import CommandJobLimits

        return CommandJobLimits(**data)


class SweepJobLimitsSchema(metaclass=PatchedSchemaMeta):
    max_concurrent_trials = fields.Int(metadata={"description": "Sweep Job max concurrent trials."})
    max_total_trials = fields.Int(
        metadata={"description": "Sweep Job max total trials."},
        required=True,
    )
    timeout = fields.Int(
        metadata={"description": "The max run duration in Seconds, after which the job will be cancelled."}
    )
    trial_timeout = fields.Int(metadata={"description": "Sweep Job Trial timeout value."})

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.sweep import SweepJobLimits

        return SweepJobLimits(**data)


class DoWhileLimitsSchema(metaclass=PatchedSchemaMeta):
    max_iteration_count = fields.Int(
        metadata={"description": "The max iteration for do_while loop."},
        validate=validate.Range(min=1, max=1000),
        required=True,
    )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import logging

from marshmallow import fields, validate

from azure.ai.ml._restclient.v2022_02_01_preview.models import EndpointAuthMode
from azure.ai.ml._schema.core.fields import NestedField, StringTransformedEnum
from azure.ai.ml._schema.core.schema import PathAwareSchema
from azure.ai.ml._schema.identity import IdentitySchema
from azure.ai.ml._utils.utils import camel_to_snake
from azure.ai.ml.constants._endpoint import EndpointConfigurations

module_logger = logging.getLogger(__name__)


class EndpointSchema(PathAwareSchema):
    id = fields.Str()
    name = fields.Str(required=True, validate=validate.Regexp(EndpointConfigurations.NAME_REGEX_PATTERN))
    description = fields.Str(metadata={"description": "Description of the inference endpoint."})
    tags = fields.Dict()
    provisioning_state = fields.Str(metadata={"description": "Provisioning state for the endpoint."})
    properties = fields.Dict()
    auth_mode = StringTransformedEnum(
        allowed_values=[
            EndpointAuthMode.AML_TOKEN,
            EndpointAuthMode.KEY,
            EndpointAuthMode.AAD_TOKEN,
        ],
        casing_transform=camel_to_snake,
        metadata={
            "description": """authentication method: no auth, key based or azure ml token based.
            aad_token is only valid for batch endpoint."""
        },
    )
    scoring_uri = fields.Str(metadata={"description": "The endpoint uri that can be used for scoring"})
    location = fields.Str()
    openapi_uri = fields.Str(metadata={"description": "Endpoint Open API URI."})
    identity = NestedField(IdentitySchema)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access

from typing import Any, Dict, Optional, Union

from typing_extensions import Literal

from azure.ai.ml._restclient.v2023_06_01_preview.models import AzMonMonitoringAlertNotificationSettings
from azure.ai.ml._restclient.v2023_06_01_preview.models import MonitorDefinition as RestMonitorDefinition
from azure.ai.ml.constants._monitoring import (
    AZMONITORING,
    DEFAULT_DATA_DRIFT_SIGNAL_NAME,
    DEFAULT_DATA_QUALITY_SIGNAL_NAME,
    DEFAULT_PREDICTION_DRIFT_SIGNAL_NAME,
    DEFAULT_TOKEN_USAGE_SIGNAL_NAME,
    MonitorTargetTasks,
)
from azure.ai.ml.entities._mixins import RestTranslatableMixin
from azure.ai.ml.entities._monitoring.alert_notification import AlertNotification
from azure.ai.ml.entities._monitoring.compute import ServerlessSparkCompute
from azure.ai.ml.entities._monitoring.signals import (
    CustomMonitoringSignal,
    DataDriftSignal,
    DataQualitySignal,
    FeatureAttributionDriftSignal,
    GenerationSafetyQualitySignal,
    GenerationTokenStatisticsSignal,
    MonitoringSignal,
    PredictionDriftSignal,
)
from azure.ai.ml.entities._monitoring.target import MonitoringTarget


class MonitorDefinition(RestTranslatableMixin):
    """Monitor definition

    :keyword compute: The Spark resource configuration to be associated with the monitor
    :paramtype compute: ~azure.ai.ml.entities.SparkResourceConfiguration
    :keyword monitoring_target: The ARM ID object associated with the model or deployment that is being monitored.
    :paramtype monitoring_target: Optional[~azure.ai.ml.entities.MonitoringTarget]
    :keyword monitoring_signals: The dictionary of signals to monitor. The key is the name of the signal and the value
        is the DataSignal object. Accepted values for the DataSignal objects are DataDriftSignal, DataQualitySignal,
        PredictionDriftSignal, FeatureAttributionDriftSignal, and CustomMonitoringSignal.
    :paramtype monitoring_signals: Optional[Dict[str, Union[~azure.ai.ml.entities.DataDriftSignal
        , ~azure.ai.ml.entities.DataQualitySignal, ~azure.ai.ml.entities.PredictionDriftSignal
        , ~azure.ai.ml.entities.FeatureAttributionDriftSignal
        , ~azure.ai.ml.entities.CustomMonitoringSignal
        , ~azure.ai.ml.entities.GenerationSafetyQualitySignal
        , ~azure.ai.ml.entities.GenerationTokenStatisticsSignal
        , ~azure.ai.ml.entities.ModelPerformanceSignal]]]
    :keyword alert_notification: The alert configuration for the monitor.
    :paramtype alert_notification: Optional[Union[Literal['azmonitoring'], ~azure.ai.ml.entities.AlertNotification]]

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_spark_configurations.py
            :start-after: [START spark_monitor_definition]
            :end-before: [END spark_monitor_definition]
            :language: python
            :dedent: 8
            :caption: Creating Monitor definition.
    """

    def __init__(
        self,
        *,
        compute: ServerlessSparkCompute,
        monitoring_target: Optional[MonitoringTarget] = None,
        monitoring_signals: Dict[
            str,
            Union[
                DataDriftSignal,
                DataQualitySignal,
                PredictionDriftSignal,
                FeatureAttributionDriftSignal,
                CustomMonitoringSignal,
                GenerationSafetyQualitySignal,
                GenerationTokenStatisticsSignal,
            ],
        ] = None,  # type: ignore[assignment]
        alert_notification: Optional[Union[Literal["azmonitoring"], AlertNotification]] = None,
    ) -> None:
        self.compute = compute
        self.monitoring_target = monitoring_target
        self.monitoring_signals = monitoring_signals
        self.alert_notification = alert_notification

    def _to_rest_object(self, **kwargs: Any) -> RestMonitorDefinition:
        default_data_window_size = kwargs.get("default_data_window_size")
        ref_data_window_size = kwargs.get("ref_data_window_size")
        rest_alert_notification = None
        if self.alert_notification:
            if isinstance(self.alert_notification, str) and self.alert_notification.lower() == AZMONITORING:
                rest_alert_notification = AzMonMonitoringAlertNotificationSettings()
            else:
                if not isinstance(self.alert_notification, str):
                    rest_alert_notification = self.alert_notification._to_rest_object()

        if self.monitoring_signals is not None:
            _signals = {
                signal_name: signal._to_rest_object(
                    default_data_window_size=default_data_window_size,
                    ref_data_window_size=ref_data_window_size,
                )
                for signal_name, signal in self.monitoring_signals.items()
            }
        return RestMonitorDefinition(
            compute_configuration=self.compute._to_rest_object(),
            monitoring_target=self.monitoring_target._to_rest_object() if self.monitoring_target else None,
            signals=_signals,  # pylint: disable=possibly-used-before-assignment
            alert_notification_setting=rest_alert_notification,
        )

    @classmethod
    def _from_rest_object(
        cls,  # pylint: disable=unused-argument
        obj: RestMonitorDefinition,
        **kwargs: Any,
    ) -> "MonitorDefinition":
        from_rest_alert_notification: Any = None
        if obj.alert_notification_setting:
            if isinstance(obj.alert_notification_setting, AzMonMonitoringAlertNotificationSettings):
                from_rest_alert_notification = AZMONITORING
            else:
                from_rest_alert_notification = AlertNotification._from_rest_object(obj.alert_notification_setting)

        _monitoring_signals = {}
        for signal_name, signal in obj.signals.items():
            _monitoring_signals[signal_name] = MonitoringSignal._from_rest_object(signal)

        return cls(
            compute=ServerlessSparkCompute._from_rest_object(obj.compute_configuration),
            monitoring_target=(
                MonitoringTarget(
                    endpoint_deployment_id=obj.monitoring_target.deployment_id, ml_task=obj.monitoring_target.task_type
                )
                if obj.monitoring_target
                else None
            ),
            monitoring_signals=_monitoring_signals,  # type: ignore[arg-type]
            alert_notification=from_rest_alert_notification,
        )

    def _populate_default_signal_information(self) -> None:
        if (
            isinstance(self.monitoring_target, MonitoringTarget)
            and self.monitoring_target.ml_task is not None
            and self.monitoring_target.ml_task.lower()
            == MonitorTargetTasks.QUESTION_ANSWERING.lower()  # type: ignore[union-attr]
        ):
            self.monitoring_signals = {
                DEFAULT_TOKEN_USAGE_SIGNAL_NAME: GenerationTokenStatisticsSignal._get_default_token_statistics_signal(),
            }
        else:
            self.monitoring_signals = {
                DEFAULT_DATA_DRIFT_SIGNAL_NAME: DataDriftSignal._get_default_data_drift_signal(),
                DEFAULT_PREDICTION_DRIFT_SIGNAL_NAME: PredictionDriftSignal._get_default_prediction_drift_signal(),
                DEFAULT_DATA_QUALITY_SIGNAL_NAME: DataQualitySignal._get_default_data_quality_signal(),
            }



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from typing import Optional

from azure.ai.ml._restclient.v2023_04_01_preview.models import NlpFixedParameters as RestNlpFixedParameters
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class NlpFixedParameters(RestTranslatableMixin):
    """Configuration of fixed parameters for all candidates of an AutoML NLP Job

    :param gradient_accumulation_steps: number of steps over which to accumulate gradients before a backward
            pass. This must be a positive integer, defaults to None
    :type gradient_accumulation_steps: Optional[int]
    :param learning_rate: initial learning rate. Must be a float in (0, 1), defaults to None
    :type learning_rate: Optional[float]
    :param learning_rate_scheduler: the type of learning rate scheduler. Must choose from 'linear', 'cosine',
            'cosine_with_restarts', 'polynomial', 'constant', and 'constant_with_warmup', defaults to None
    :type learning_rate_scheduler: Optional[str]
    :param model_name: the model name to use during training. Must choose from 'bert-base-cased',
            'bert-base-uncased', 'bert-base-multilingual-cased', 'bert-base-german-cased', 'bert-large-cased',
            'bert-large-uncased', 'distilbert-base-cased', 'distilbert-base-uncased', 'roberta-base', 'roberta-large',
            'distilroberta-base', 'xlm-roberta-base', 'xlm-roberta-large', xlnet-base-cased', and 'xlnet-large-cased',
            defaults to None
    :type model_name: Optional[str]
    :param number_of_epochs: the number of epochs to train with. Must be a positive integer, defaults to None
    :type number_of_epochs: Optional[int]
    :param training_batch_size: the batch size during training. Must be a positive integer, defaults to None
    :type training_batch_size: Optional[int]
    :param validation_batch_size: the batch size during validation. Must be a positive integer, defaults to None
    :type validation_batch_size: Optional[int]
    :param warmup_ratio: ratio of total training steps used for a linear warmup from 0 to learning_rate.
            Must be a float in [0, 1], defaults to None
    :type warmup_ratio: Optional[float]
    :param weight_decay: value of weight decay when optimizer is sgd, adam, or adamw. This must be a float in
            the range [0, 1] defaults to None
    :type weight_decay: Optional[float]

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_automl_nlp.py
                :start-after: [START automl.nlp_fixed_parameters]
                :end-before: [END automl.nlp_fixed_parameters]
                :language: python
                :dedent: 8
                :caption: creating an nlp fixed parameters
    """

    def __init__(
        self,
        *,
        gradient_accumulation_steps: Optional[int] = None,
        learning_rate: Optional[float] = None,
        learning_rate_scheduler: Optional[str] = None,
        model_name: Optional[str] = None,
        number_of_epochs: Optional[int] = None,
        training_batch_size: Optional[int] = None,
        validation_batch_size: Optional[int] = None,
        warmup_ratio: Optional[float] = None,
        weight_decay: Optional[float] = None,
    ):
        self.gradient_accumulation_steps = gradient_accumulation_steps
        self.learning_rate = learning_rate
        self.learning_rate_scheduler = learning_rate_scheduler
        self.model_name = model_name
        self.number_of_epochs = number_of_epochs
        self.training_batch_size = training_batch_size
        self.validation_batch_size = validation_batch_size
        self.warmup_ratio = warmup_ratio
        self.weight_decay = weight_decay

    def _to_rest_object(self) -> RestNlpFixedParameters:
        return RestNlpFixedParameters(
            gradient_accumulation_steps=self.gradient_accumulation_steps,
            learning_rate=self.learning_rate,
            learning_rate_scheduler=self.learning_rate_scheduler,
            model_name=self.model_name,
            number_of_epochs=self.number_of_epochs,
            training_batch_size=self.training_batch_size,
            validation_batch_size=self.validation_batch_size,
            warmup_ratio=self.warmup_ratio,
            weight_decay=self.weight_decay,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestNlpFixedParameters) -> "NlpFixedParameters":
        return cls(
            gradient_accumulation_steps=obj.gradient_accumulation_steps,
            learning_rate=obj.learning_rate,
            learning_rate_scheduler=obj.learning_rate_scheduler,
            model_name=obj.model_name,
            number_of_epochs=obj.number_of_epochs,
            training_batch_size=obj.training_batch_size,
            validation_batch_size=obj.validation_batch_size,
            warmup_ratio=obj.warmup_ratio,
            weight_decay=obj.weight_decay,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, NlpFixedParameters):
            return NotImplemented

        return (
            self.gradient_accumulation_steps == other.gradient_accumulation_steps
            and self.learning_rate == other.learning_rate
            and self.learning_rate_scheduler == other.learning_rate_scheduler
            and self.model_name == other.model_name
            and self.number_of_epochs == other.number_of_epochs
            and self.training_batch_size == other.training_batch_size
            and self.validation_batch_size == other.validation_batch_size
            and self.warmup_ratio == other.warmup_ratio
            and self.weight_decay == other.weight_decay
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



import pytest

from azure.ai.ml._utils.utils import from_iso_duration_format_min_sec


@pytest.mark.unittest
@pytest.mark.core_sdk_test
class TestUtils:
    def test_from_iso_duration_format_min_sec(self) -> None:
        duration = from_iso_duration_format_min_sec("PT10M41.7894561S")
        assert duration == "10m 41s"



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from typing import Any

from azure.ai.ml._schema.component.input_output import SUPPORTED_PARAM_TYPES
from azure.ai.ml.entities._mixins import DictMixin, RestTranslatableMixin


class _InputOutputBase(DictMixin, RestTranslatableMixin):
    def __init__(
        self,
        *,
        # pylint: disable=redefined-builtin
        type: Any,
        # pylint: disable=unused-argument
        **kwargs: Any,
    ) -> None:
        """Base class for Input & Output class.

        This class is introduced to support literal output in the future.

        :param type: The type of the Input/Output.
        :type type: str
        """
        self.type = type

    def _is_literal(self) -> bool:
        """Check whether input is a literal

        :return: True if this input is literal input.
        :rtype: bool
        """
        return self.type in SUPPORTED_PARAM_TYPES



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._batch_endpoints_operations import BatchEndpointsOperations
from ._batch_deployments_operations import BatchDeploymentsOperations
from ._code_containers_operations import CodeContainersOperations
from ._code_versions_operations import CodeVersionsOperations
from ._component_containers_operations import ComponentContainersOperations
from ._component_versions_operations import ComponentVersionsOperations
from ._data_containers_operations import DataContainersOperations
from ._data_versions_operations import DataVersionsOperations
from ._datastores_operations import DatastoresOperations
from ._environment_containers_operations import EnvironmentContainersOperations
from ._environment_versions_operations import EnvironmentVersionsOperations
from ._jobs_operations import JobsOperations
from ._model_containers_operations import ModelContainersOperations
from ._model_versions_operations import ModelVersionsOperations
from ._online_endpoints_operations import OnlineEndpointsOperations
from ._online_deployments_operations import OnlineDeploymentsOperations
from ._operations import Operations
from ._workspaces_operations import WorkspacesOperations
from ._usages_operations import UsagesOperations
from ._virtual_machine_sizes_operations import VirtualMachineSizesOperations
from ._quotas_operations import QuotasOperations
from ._compute_operations import ComputeOperations
from ._private_endpoint_connections_operations import PrivateEndpointConnectionsOperations
from ._private_link_resources_operations import PrivateLinkResourcesOperations
from ._workspace_connections_operations import WorkspaceConnectionsOperations
from ._workspace_features_operations import WorkspaceFeaturesOperations

__all__ = [
    'BatchEndpointsOperations',
    'BatchDeploymentsOperations',
    'CodeContainersOperations',
    'CodeVersionsOperations',
    'ComponentContainersOperations',
    'ComponentVersionsOperations',
    'DataContainersOperations',
    'DataVersionsOperations',
    'DatastoresOperations',
    'EnvironmentContainersOperations',
    'EnvironmentVersionsOperations',
    'JobsOperations',
    'ModelContainersOperations',
    'ModelVersionsOperations',
    'OnlineEndpointsOperations',
    'OnlineDeploymentsOperations',
    'Operations',
    'WorkspacesOperations',
    'UsagesOperations',
    'VirtualMachineSizesOperations',
    'QuotasOperations',
    'ComputeOperations',
    'PrivateEndpointConnectionsOperations',
    'PrivateLinkResourcesOperations',
    'WorkspaceConnectionsOperations',
    'WorkspaceFeaturesOperations',
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

import logging

from marshmallow import ValidationError, fields, post_load, pre_dump

from azure.ai.ml._restclient.v2022_02_01_preview.models import EarlyTerminationPolicyType
from azure.ai.ml._schema.core.fields import StringTransformedEnum
from azure.ai.ml._schema.core.schema import PatchedSchemaMeta
from azure.ai.ml._utils.utils import camel_to_snake

module_logger = logging.getLogger(__name__)


class EarlyTerminationPolicySchema(metaclass=PatchedSchemaMeta):
    evaluation_interval = fields.Int(allow_none=True)
    delay_evaluation = fields.Int(allow_none=True)


class BanditPolicySchema(EarlyTerminationPolicySchema):
    type = StringTransformedEnum(
        required=True,
        allowed_values=EarlyTerminationPolicyType.BANDIT,
        casing_transform=camel_to_snake,
    )
    slack_factor = fields.Float(allow_none=True)
    slack_amount = fields.Float(allow_none=True)

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.sweep import BanditPolicy

        data.pop("type", None)
        return BanditPolicy(**data)

    @pre_dump
    def predump(self, data, **kwargs):
        from azure.ai.ml.sweep import BanditPolicy

        if not isinstance(data, BanditPolicy):
            raise ValidationError("Cannot dump non-BanditPolicy object into BanditPolicySchema")
        return data


class MedianStoppingPolicySchema(EarlyTerminationPolicySchema):
    type = StringTransformedEnum(
        required=True,
        allowed_values=EarlyTerminationPolicyType.MEDIAN_STOPPING,
        casing_transform=camel_to_snake,
    )

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.sweep import MedianStoppingPolicy

        data.pop("type", None)
        return MedianStoppingPolicy(**data)

    @pre_dump
    def predump(self, data, **kwargs):
        from azure.ai.ml.sweep import MedianStoppingPolicy

        if not isinstance(data, MedianStoppingPolicy):
            raise ValidationError("Cannot dump non-MedicanStoppingPolicy object into MedianStoppingPolicySchema")
        return data


class TruncationSelectionPolicySchema(EarlyTerminationPolicySchema):
    type = StringTransformedEnum(
        required=True,
        allowed_values=EarlyTerminationPolicyType.TRUNCATION_SELECTION,
        casing_transform=camel_to_snake,
    )
    truncation_percentage = fields.Int(required=True)

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.sweep import TruncationSelectionPolicy

        data.pop("type", None)
        return TruncationSelectionPolicy(**data)

    @pre_dump
    def predump(self, data, **kwargs):
        from azure.ai.ml.sweep import TruncationSelectionPolicy

        if not isinstance(data, TruncationSelectionPolicy):
            raise ValidationError(
                "Cannot dump non-TruncationSelectionPolicy object into TruncationSelectionPolicySchema"
            )
        return data



import pytest

from azure.ai.ml import load_component
from azure.ai.ml.entities import Component as ComponentEntity
from azure.ai.ml.entities import PipelineJobSettings
from azure.ai.ml.entities._builders import Command
from azure.ai.ml.entities._job.pipeline._attr_dict import _AttrDict

from .._util import _DSL_TIMEOUT_SECOND


@pytest.mark.timeout(_DSL_TIMEOUT_SECOND)
@pytest.mark.unittest
@pytest.mark.pipeline_test
class TestAttrDict:
    def test_attr_dict(self):
        obj = _AttrDict()
        obj.resource_layout.node_count = 2
        obj.resource_layout.process_count_per_node = 2
        # setting same filed twice will return same object
        assert obj.resource_layout == {"node_count": 2, "process_count_per_node": 2}
        # setting internal attribute will not be arbitrary attribute
        obj._target = "target"

        obj.target = "aml"
        # all set attribute should be recorded
        assert obj._get_attrs() == {"resource_layout": {"node_count": 2, "process_count_per_node": 2}, "target": "aml"}
        # getting non-exist attribute will return an empty dict
        assert obj.sweep == {}

        # only calling existing methods in object is allowed
        with pytest.raises(TypeError):
            obj.resource_layout()

        # only setting resource_layout.node_count and resource_layout.process_count_per_node is allowed
        obj = _AttrDict(allowed_keys={"resource_layout": {"node_count": None, "process_count_per_node": None}})
        obj.resource_layout.node_count = 2
        obj.resource_layout.process_count_per_node = 2
        assert obj.resource_layout == {"node_count": 2, "process_count_per_node": 2}

        # target will not be arbitrary attribute
        obj.target = "aml"
        assert obj._get_attrs() == {"resource_layout": {"node_count": 2, "process_count_per_node": 2}}
        assert "target" in obj.__dict__.keys()
        assert "target" not in obj.items()

        # set/get item is not supported
        # currently we use copy.copy to dump a pipeline/component, which depends on __setitem__
        # disable this check for now
        # with pytest.raises(UnsupportedOperationError) as e:
        #     obj["target"] = "aml"
        # assert "Operation __setitem__ is not supported." in str(e)

        # with pytest.raises(UnsupportedOperationError) as e:
        #     obj.resource_layout = obj["target"]
        # assert "Operation __getitem__ is not supported." in str(e)

    def test_allowed_keys_unsupported_type(self):
        # Only dict and None are supported as allowed_keys
        with pytest.raises(TypeError):
            _AttrDict(allowed_keys=object())

    def test_component_attr_dict(self):
        command_component = load_component(source="./tests/test_configs/components/helloworld_component.yml")
        node = Command(component=command_component)
        node.resource_layout.node_count = 2
        node.resource_layout.process_count_per_node = 2
        assert node.resource_layout == {"node_count": 2, "process_count_per_node": 2}
        # compute is already an attribute, settings it won't change arbitrary dict.
        node.compute = "aml"
        assert not isinstance(node.compute, _AttrDict)
        assert node._get_attrs() == {"resource_layout": {"node_count": 2, "process_count_per_node": 2}}

    def test_attr_dict_as_bool(self):
        obj = _AttrDict()
        assert not obj
        obj.resource_layout.node_count = 2
        assert obj

        obj = PipelineJobSettings()
        assert not obj
        obj.continue_on_step_failure = False
        assert obj

    def test_attr_dict_false_value(self):
        obj = _AttrDict()
        obj.false_value = False
        assert obj._get_attrs() == {"false_value": False}



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import json
from typing import Any, Dict, List, Optional

from azure.ai.ml._restclient.v2024_10_01_preview.models import (
    DiagnoseRequestProperties as RestDiagnoseRequestProperties,
)
from azure.ai.ml._restclient.v2024_10_01_preview.models import DiagnoseResponseResult as RestDiagnoseResponseResult
from azure.ai.ml._restclient.v2024_10_01_preview.models import (
    DiagnoseResponseResultValue as RestDiagnoseResponseResultValue,
)
from azure.ai.ml._restclient.v2024_10_01_preview.models import DiagnoseResult as RestDiagnoseResult
from azure.ai.ml._restclient.v2024_10_01_preview.models import (
    DiagnoseWorkspaceParameters as RestDiagnoseWorkspaceParameters,
)


class DiagnoseRequestProperties:
    """DiagnoseRequestProperties."""

    def __init__(
        self,
        *,
        udr: Optional[Dict[str, Any]] = None,
        nsg: Optional[Dict[str, Any]] = None,
        resource_lock: Optional[Dict[str, Any]] = None,
        dns_resolution: Optional[Dict[str, Any]] = None,
        storage_account: Optional[Dict[str, Any]] = None,
        key_vault: Optional[Dict[str, Any]] = None,
        container_registry: Optional[Dict[str, Any]] = None,
        application_insights: Optional[Dict[str, Any]] = None,
        others: Optional[Dict[str, Any]] = None,
    ):
        self.udr = udr
        self.nsg = nsg
        self.resource_lock = resource_lock
        self.dns_resolution = dns_resolution
        self.storage_account = storage_account
        self.key_vault = key_vault
        self.container_registry = container_registry
        self.application_insights = application_insights
        self.others = others

    @classmethod
    def _from_rest_object(cls, rest_obj: RestDiagnoseRequestProperties) -> "DiagnoseRequestProperties":
        return cls(
            udr=rest_obj.udr,
            nsg=rest_obj.nsg,
            resource_lock=rest_obj.resource_lock,
            dns_resolution=rest_obj.dns_resolution,
            storage_account=rest_obj.storage_account,
            key_vault=rest_obj.key_vault,
            container_registry=rest_obj.container_registry,
            application_insights=rest_obj.application_insights,
            others=rest_obj.others,
        )

    def _to_rest_object(self) -> RestDiagnoseRequestProperties:
        return RestDiagnoseRequestProperties(
            udr=self.udr,
            nsg=self.nsg,
            resource_lock=self.resource_lock,
            dns_resolution=self.dns_resolution,
            storage_account=self.storage_account,
            key_vault=self.key_vault,
            container_registry=self.container_registry,
            application_insights=self.application_insights,
            others=self.others,
        )


class DiagnoseResponseResult:
    """DiagnoseResponseResult."""

    def __init__(
        self,
        *,
        value: Optional["DiagnoseResponseResultValue"] = None,
    ):
        self.value = value

    @classmethod
    def _from_rest_object(cls, rest_obj: RestDiagnoseResponseResult) -> "DiagnoseResponseResult":
        val = None
        if rest_obj and rest_obj.value and isinstance(rest_obj.value, RestDiagnoseResponseResultValue):
            # pylint: disable=protected-access
            val = DiagnoseResponseResultValue._from_rest_object(rest_obj.value)
        return cls(value=val)

    def _to_rest_object(self) -> RestDiagnoseResponseResult:
        return RestDiagnoseResponseResult(value=self.value)


class DiagnoseResponseResultValue:
    """DiagnoseResponseResultValue."""

    def __init__(
        self,
        *,
        user_defined_route_results: Optional[List["DiagnoseResult"]] = None,
        network_security_rule_results: Optional[List["DiagnoseResult"]] = None,
        resource_lock_results: Optional[List["DiagnoseResult"]] = None,
        dns_resolution_results: Optional[List["DiagnoseResult"]] = None,
        storage_account_results: Optional[List["DiagnoseResult"]] = None,
        key_vault_results: Optional[List["DiagnoseResult"]] = None,
        container_registry_results: Optional[List["DiagnoseResult"]] = None,
        application_insights_results: Optional[List["DiagnoseResult"]] = None,
        other_results: Optional[List["DiagnoseResult"]] = None,
    ):
        self.user_defined_route_results = user_defined_route_results
        self.network_security_rule_results = network_security_rule_results
        self.resource_lock_results = resource_lock_results
        self.dns_resolution_results = dns_resolution_results
        self.storage_account_results = storage_account_results
        self.key_vault_results = key_vault_results
        self.container_registry_results = container_registry_results
        self.application_insights_results = application_insights_results
        self.other_results = other_results

    @classmethod
    def _from_rest_object(cls, rest_obj: RestDiagnoseResponseResultValue) -> "DiagnoseResponseResultValue":
        return cls(
            user_defined_route_results=rest_obj.user_defined_route_results,
            network_security_rule_results=rest_obj.network_security_rule_results,
            resource_lock_results=rest_obj.resource_lock_results,
            dns_resolution_results=rest_obj.dns_resolution_results,
            storage_account_results=rest_obj.storage_account_results,
            key_vault_results=rest_obj.key_vault_results,
            container_registry_results=rest_obj.container_registry_results,
            application_insights_results=rest_obj.application_insights_results,
            other_results=rest_obj.other_results,
        )

    def _to_rest_object(self) -> RestDiagnoseResponseResultValue:
        return RestDiagnoseResponseResultValue(
            user_defined_route_results=self.user_defined_route_results,
            network_security_rule_results=self.network_security_rule_results,
            resource_lock_results=self.resource_lock_results,
            dns_resolution_results=self.dns_resolution_results,
            storage_account_results=self.storage_account_results,
            key_vault_results=self.key_vault_results,
            container_registry_results=self.container_registry_results,
            application_insights_results=self.application_insights_results,
            other_results=self.other_results,
        )

    def __json__(self):
        results = self.__dict__.copy()
        for k, v in results.items():
            results[k] = [item.__dict__ for item in v]
        return results

    def __str__(self) -> str:
        return json.dumps(self, default=lambda o: o.__json__(), indent=2)


class DiagnoseResult:
    """Result of Diagnose."""

    def __init__(
        self,
        *,
        code: Optional[str] = None,
        level: Optional[str] = None,
        message: Optional[str] = None,
    ):
        self.code = code
        self.level = level
        self.message = message

    @classmethod
    def _from_rest_object(cls, rest_obj: RestDiagnoseResult) -> "DiagnoseResult":
        return cls(
            code=rest_obj.code,
            level=rest_obj.level,
            message=rest_obj.message,
        )

    def _to_rest_object(self) -> RestDiagnoseResult:
        return RestDiagnoseResult(
            code=self.code,
            level=self.level,
            message=self.message,
        )


class DiagnoseWorkspaceParameters:
    """Parameters to diagnose a workspace."""

    def __init__(
        self,
        *,
        value: Optional["DiagnoseRequestProperties"] = None,
    ):
        self.value = value

    @classmethod
    def _from_rest_object(cls, rest_obj: RestDiagnoseWorkspaceParameters) -> "DiagnoseWorkspaceParameters":
        val = None
        if rest_obj.value and isinstance(rest_obj.value, DiagnoseRequestProperties):
            # TODO: Bug Item number: 2883283
            # pylint: disable=protected-access
            val = rest_obj.value._from_rest_object()  # type: ignore
        return cls(value=val)

    def _to_rest_object(self) -> RestDiagnoseWorkspaceParameters:
        val = None
        if self.value and isinstance(self.value, DiagnoseRequestProperties):
            # pylint: disable=protected-access
            val = self.value._to_rest_object()
        return RestDiagnoseWorkspaceParameters(value=val)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

from marshmallow import fields, post_load

from azure.ai.ml._schema import ExperimentalField
from azure.ai.ml._schema.core.schema import PatchedSchemaMeta
from azure.ai.ml.constants._job.automl import AutoMLConstants


class AutoMLTableLimitsSchema(metaclass=PatchedSchemaMeta):
    enable_early_termination = fields.Bool()
    exit_score = fields.Float()
    max_concurrent_trials = fields.Int()
    max_cores_per_trial = fields.Int()
    max_nodes = ExperimentalField(fields.Int())
    max_trials = fields.Int(data_key=AutoMLConstants.MAX_TRIALS_YAML)
    timeout_minutes = fields.Int()  # type duration
    trial_timeout_minutes = fields.Int()  # type duration

    @post_load
    def make(self, data, **kwargs) -> "TabularLimitSettings":
        from azure.ai.ml.automl import TabularLimitSettings

        return TabularLimitSettings(**data)



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import Any, Callable, Dict, Optional, TypeVar

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._private_link_resources_operations import build_list_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class PrivateLinkResourcesOperations:
    """PrivateLinkResourcesOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.machinelearningservices.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace_async
    async def list(
        self,
        resource_group_name: str,
        workspace_name: str,
        **kwargs: Any
    ) -> "_models.PrivateLinkResourceListResult":
        """Gets the private link resources that need to be created for a workspace.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace.
        :type workspace_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: PrivateLinkResourceListResult, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.PrivateLinkResourceListResult
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PrivateLinkResourceListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-10-01-preview")  # type: str

        
        request = build_list_request(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            api_version=api_version,
            template_url=self.list.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('PrivateLinkResourceListResult', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/privateLinkResources"}  # type: ignore




# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

import logging

from marshmallow import fields, post_load

from azure.ai.ml._restclient.v2022_10_01.models import Goal
from azure.ai.ml._schema.core.fields import StringTransformedEnum
from azure.ai.ml._schema.core.schema import PatchedSchemaMeta
from azure.ai.ml._utils.utils import camel_to_snake

module_logger = logging.getLogger(__name__)


class SweepObjectiveSchema(metaclass=PatchedSchemaMeta):
    goal = StringTransformedEnum(
        required=True,
        allowed_values=[Goal.MINIMIZE, Goal.MAXIMIZE],
        casing_transform=camel_to_snake,
    )
    primary_metric = fields.Str(required=True)

    @post_load
    def make(self, data, **kwargs) -> "Objective":
        from azure.ai.ml.entities._job.sweep.objective import Objective

        return Objective(**data)



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: ml_samples_automl_image.py
DESCRIPTION:
    These samples demonstrate how to use AutoML Image functions
USAGE:
    python ml_samples_automl_image.py

"""


class AutoMLImageSamples(object):
    def automl_image_jobs(self):

        # [START automl.image_classification]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes
        from azure.ai.ml.automl import ClassificationMultilabelPrimaryMetrics

        image_classification_job = automl.image_classification(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="label",
            primary_metric=ClassificationMultilabelPrimaryMetrics.ACCURACY,
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.image_classification]

        # [START automl.image_classification_multilabel]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes
        from azure.ai.ml.automl import ClassificationMultilabelPrimaryMetrics

        image_classification_multilabel_job = automl.image_classification_multilabel(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="label",
            primary_metric=ClassificationMultilabelPrimaryMetrics.IOU,
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.image_classification_multilabel]

        # [START automl.image_object_detection]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes
        from azure.ai.ml.automl import ObjectDetectionPrimaryMetrics

        image_object_detection_job = automl.image_object_detection(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="label",
            primary_metric=ObjectDetectionPrimaryMetrics.MEAN_AVERAGE_PRECISION,
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.image_object_detection]

        # [START automl.image_instance_segmentation]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes
        from azure.ai.ml.automl import InstanceSegmentationPrimaryMetrics

        image_instance_segmentation_job = automl.image_instance_segmentation(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="label",
            primary_metric=InstanceSegmentationPrimaryMetrics.MEAN_AVERAGE_PRECISION,
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.image_instance_segmentation]

        # [START automl.automl_image_job.image_classification_job]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes
        from azure.ai.ml.automl import ClassificationMultilabelPrimaryMetrics

        image_classification_job = automl.ImageClassificationJob(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="label",
            primary_metric=ClassificationMultilabelPrimaryMetrics.ACCURACY,
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.automl_image_job.image_classification_job]

        # [START automl.automl_image_job.image_classification_multilabel_job]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes
        from azure.ai.ml.automl import ClassificationMultilabelPrimaryMetrics

        image_classification_multilabel_job = automl.ImageClassificationMultilabelJob(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="terms",
            primary_metric=ClassificationMultilabelPrimaryMetrics.IOU,
            tags={"my_custom_tag": "My custom value"},
        )
        # [END automl.automl_image_job.image_classification_multilabel_job]

        # [START automl.automl_image_job.image_object_detection_job]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes
        from azure.ai.ml.automl import ObjectDetectionPrimaryMetrics

        image_object_detection_job = automl.ImageObjectDetectionJob(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            tags={"my_custom_tag": "My custom value"},
            primary_metric=ObjectDetectionPrimaryMetrics.MEAN_AVERAGE_PRECISION,
        )
        # [END automl.automl_image_job.image_object_detection_job]

        # [START automl.automl_image_job.image_instance_segmentation_job]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes
        from azure.ai.ml.automl import ObjectDetectionPrimaryMetrics

        image_instance_segmentation_job = automl.ImageInstanceSegmentationJob(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            tags={"my_custom_tag": "My custom value"},
            primary_metric=ObjectDetectionPrimaryMetrics.MEAN_AVERAGE_PRECISION,
        )
        # [END automl.automl_image_job.image_instance_segmentation_job]

        # [START automl.automl_image_job.image_sweep_settings]
        from azure.ai.ml import automl
        from azure.ai.ml.sweep import BanditPolicy

        image_sweep_settings = automl.ImageSweepSettings(
            sampling_algorithm="Grid",
            early_termination=BanditPolicy(evaluation_interval=2, slack_factor=0.05, delay_evaluation=6),
        )
        # [END automl.automl_image_job.image_sweep_settings]

        # [START automl.automl_image_job.image_classification_search_space]
        from azure.ai.ml import automl
        from azure.ai.ml.sweep import Uniform, Choice

        image_classification_search_space = automl.ImageClassificationSearchSpace(
            model_name="vitb16r224",
            number_of_epochs=Choice([15, 30]),
            weight_decay=Uniform(0.01, 0.1),
        )
        # [END automl.automl_image_job.image_classification_search_space]

        # [START automl.automl_image_job.image_object_detection_search_space]
        from azure.ai.ml import automl
        from azure.ai.ml.sweep import Uniform

        image_detection_search_space = automl.ImageObjectDetectionSearchSpace(
            learning_rate=Uniform(0.005, 0.05),
            model_name="yolov5",
            weight_decay=Uniform(0.01, 0.1),
        )
        # [END automl.automl_image_job.image_object_detection_search_space]

        # [START automl.automl_image_job.image_limit_settings]
        from azure.ai.ml import automl, Input
        from azure.ai.ml.constants import AssetTypes

        # Create the AutoML job with the related factory-function.
        image_job = automl.image_instance_segmentation(
            experiment_name="my_experiment",
            compute="my_compute",
            training_data=Input(type=AssetTypes.MLTABLE, path="./training-mltable-folder"),
            validation_data=Input(type=AssetTypes.MLTABLE, path="./validation-mltable-folder"),
            target_column_name="label",
            primary_metric="MeanAveragePrecision",
            tags={"my_custom_tag": "custom value"},
        )
        # Set the limits for the AutoML job.
        image_job.set_limits(
            max_trials=10,
            max_concurrent_trials=2,
        )
        # [END automl.automl_image_job.image_limit_settings]

        # [START automl.automl_image_job.image_classification_model_settings]
        from azure.ai.ml import automl

        image_classification_model_settings = automl.ImageModelSettingsClassification(
            checkpoint_frequency=5,
            early_stopping=False,
            gradient_accumulation_step=2,
        )
        # [END automl.automl_image_job.image_classification_model_settings]

        # [START automl.automl_image_job.image_object_detection_model_settings]
        from azure.ai.ml import automl

        object_detection_model_settings = automl.ImageModelSettingsObjectDetection(min_size=600, max_size=1333)
        # [END automl.automl_image_job.image_object_detection_model_settings]


if __name__ == "__main__":
    sample = AutoMLImageSamples()
    sample.automl_image_jobs()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access,too-many-instance-attributes

from typing import Any, Dict, Optional

from azure.ai.ml._restclient.v2022_12_01_preview.models import AmlCompute as AmlComputeRest
from azure.ai.ml._restclient.v2022_12_01_preview.models import (
    AmlComputeProperties,
    ComputeResource,
    ResourceId,
    ScaleSettings,
    UserAccountCredentials,
)
from azure.ai.ml._schema._utils.utils import get_subnet_str
from azure.ai.ml._schema.compute.aml_compute import AmlComputeSchema
from azure.ai.ml._utils.utils import camel_to_snake, snake_to_pascal, to_iso_duration_format
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, TYPE
from azure.ai.ml.constants._compute import ComputeDefaults, ComputeType
from azure.ai.ml.entities._credentials import IdentityConfiguration
from azure.ai.ml.entities._util import load_from_dict

from .compute import Compute, NetworkSettings


class AmlComputeSshSettings:
    """SSH settings to access a AML compute target.

    :param admin_username: SSH user name.
    :type admin_username: str
    :param admin_password: SSH user password. Defaults to None.
    :type admin_password: str
    :param ssh_key_value: The SSH RSA private key. Use "ssh-keygen -t
        rsa -b 2048" to generate your SSH key pairs. Defaults to None.
    :type ssh_key_value: Optional[str]

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_compute.py
            :start-after: [START aml_compute_ssh_settings]
            :end-before: [END aml_compute_ssh_settings]
            :language: python
            :dedent: 8
            :caption: Configuring an AmlComputeSshSettings object.
    """

    def __init__(
        self,
        *,
        admin_username: str,
        admin_password: Optional[str] = None,
        ssh_key_value: Optional[str] = None,
    ) -> None:
        self.admin_username = admin_username
        self.admin_password = admin_password
        self.ssh_key_value = ssh_key_value

    def _to_user_account_credentials(self) -> UserAccountCredentials:
        return UserAccountCredentials(
            admin_user_name=self.admin_username,
            admin_user_password=self.admin_password,
            admin_user_ssh_public_key=self.ssh_key_value,
        )

    @classmethod
    def _from_user_account_credentials(cls, credentials: UserAccountCredentials) -> "AmlComputeSshSettings":
        return cls(
            admin_username=credentials.admin_user_name,
            admin_password=credentials.admin_user_password,
            ssh_key_value=credentials.admin_user_ssh_public_key,
        )


class AmlCompute(Compute):
    """AzureML Compute resource.

    :param name: Name of the compute resource.
    :type name: str
    :param description: Description of the compute resource.
    :type description: Optional[str]
    :param size: Size of the compute. Defaults to None.
    :type size: Optional[str]
    :param tags: A set of tags. Contains resource tags defined as key/value pairs.
    :type tags: Optional[dict[str, str]]
    :param ssh_settings: SSH settings to access the AzureML compute cluster.
    :type ssh_settings: Optional[~azure.ai.ml.entities.AmlComputeSshSettings]
    :param network_settings: Virtual network settings for the AzureML compute cluster.
    :type network_settings: Optional[~azure.ai.ml.entities.NetworkSettings]
    :param idle_time_before_scale_down: Node idle time before scaling down. Defaults to None.
    :type idle_time_before_scale_down: Optional[int]
    :param identity: The identities that are associated with the compute cluster.
    :type identity: Optional[~azure.ai.ml.entities.IdentityConfiguration]
    :param tier: Virtual Machine tier. Accepted values include: "Dedicated", "LowPriority". Defaults to None.
    :type tier: Optional[str]
    :param min_instances: Minimum number of instances. Defaults to None.
    :type min_instances: Optional[int]
    :param max_instances: Maximum number of instances. Defaults to None.
    :type max_instances: Optional[int]
    :param ssh_public_access_enabled: State of the public SSH port. Accepted values are:
        * False - Indicates that the public SSH port is closed on all nodes of the cluster.
        * True - Indicates that the public SSH port is open on all nodes of the cluster.
        * None - Indicates that the public SSH port is closed on all nodes of the cluster if VNet is defined,
        else is open all public nodes.
        It can be None only during cluster creation time. After creation it will be either True or False.
        Defaults to None.
    :type ssh_public_access_enabled: Optional[bool]
    :param enable_node_public_ip: Enable or disable node public IP address provisioning. Accepted values are:
        * True - Indicates that the compute nodes will have public IPs provisioned.
        * False - Indicates that the compute nodes will have a private endpoint and no public IPs.
        Defaults to True.
    :type enable_node_public_ip: bool

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_compute.py
            :start-after: [START amlcompute]
            :end-before: [END amlcompute]
            :language: python
            :dedent: 8
            :caption: Creating an AmlCompute object.
    """

    def __init__(
        self,
        *,
        name: str,
        description: Optional[str] = None,
        size: Optional[str] = None,
        tags: Optional[dict] = None,
        ssh_public_access_enabled: Optional[bool] = None,
        ssh_settings: Optional[AmlComputeSshSettings] = None,
        min_instances: Optional[int] = None,
        max_instances: Optional[int] = None,
        network_settings: Optional[NetworkSettings] = None,
        idle_time_before_scale_down: Optional[int] = None,
        identity: Optional[IdentityConfiguration] = None,
        tier: Optional[str] = None,
        enable_node_public_ip: bool = True,
        **kwargs: Any,
    ) -> None:
        kwargs[TYPE] = ComputeType.AMLCOMPUTE
        super().__init__(
            name=name,
            description=description,
            location=kwargs.pop("location", None),
            tags=tags,
            **kwargs,
        )
        self.size = size
        self.min_instances = min_instances or 0
        self.max_instances = max_instances or 1
        self.idle_time_before_scale_down = idle_time_before_scale_down
        self.identity = identity
        self.ssh_public_access_enabled = ssh_public_access_enabled
        self.ssh_settings = ssh_settings
        self.network_settings = network_settings
        self.tier = tier
        self.enable_node_public_ip = enable_node_public_ip
        self.subnet = None

    @classmethod
    def _load_from_rest(cls, rest_obj: ComputeResource) -> "AmlCompute":
        prop = rest_obj.properties

        network_settings = None
        if prop.properties.subnet or (prop.properties.enable_node_public_ip is not None):
            network_settings = NetworkSettings(
                subnet=prop.properties.subnet.id if prop.properties.subnet else None,
            )

        ssh_settings = (
            AmlComputeSshSettings._from_user_account_credentials(prop.properties.user_account_credentials)
            if prop.properties.user_account_credentials
            else None
        )

        response = AmlCompute(
            name=rest_obj.name,
            id=rest_obj.id,
            description=prop.description,
            location=(prop.compute_location if prop.compute_location else rest_obj.location),
            tags=rest_obj.tags if rest_obj.tags else None,
            provisioning_state=prop.provisioning_state,
            provisioning_errors=(
                prop.provisioning_errors[0].error.code
                if (prop.provisioning_errors and len(prop.provisioning_errors) > 0)
                else None
            ),
            size=prop.properties.vm_size,
            tier=camel_to_snake(prop.properties.vm_priority),
            min_instances=(prop.properties.scale_settings.min_node_count if prop.properties.scale_settings else None),
            max_instances=(prop.properties.scale_settings.max_node_count if prop.properties.scale_settings else None),
            network_settings=network_settings or None,
            ssh_settings=ssh_settings,
            ssh_public_access_enabled=(prop.properties.remote_login_port_public_access == "Enabled"),
            idle_time_before_scale_down=(
                prop.properties.scale_settings.node_idle_time_before_scale_down.total_seconds()
                if prop.properties.scale_settings and prop.properties.scale_settings.node_idle_time_before_scale_down
                else None
            ),
            identity=(
                IdentityConfiguration._from_compute_rest_object(rest_obj.identity) if rest_obj.identity else None
            ),
            created_on=prop.additional_properties.get("createdOn", None),
            enable_node_public_ip=(
                prop.properties.enable_node_public_ip if prop.properties.enable_node_public_ip is not None else True
            ),
        )
        return response

    def _set_full_subnet_name(self, subscription_id: str, rg: str) -> None:
        if self.network_settings:
            self.subnet = get_subnet_str(
                self.network_settings.vnet_name,
                self.network_settings.subnet,
                subscription_id,
                rg,
            )

    def _to_dict(self) -> Dict:
        res: dict = AmlComputeSchema(context={BASE_PATH_CONTEXT_KEY: "./"}).dump(self)
        return res

    @classmethod
    def _load_from_dict(cls, data: Dict, context: Dict, **kwargs: Any) -> "AmlCompute":
        loaded_data = load_from_dict(AmlComputeSchema, data, context, **kwargs)
        return AmlCompute(**loaded_data)

    def _to_rest_object(self) -> ComputeResource:
        if self.network_settings and self.network_settings.subnet:
            subnet_resource = ResourceId(id=self.subnet)
        else:
            subnet_resource = None

        # Scale settings is required when creating an AzureML compute cluster.
        scale_settings = ScaleSettings(
            max_node_count=self.max_instances,
            min_node_count=self.min_instances,
            node_idle_time_before_scale_down=(
                to_iso_duration_format(int(self.idle_time_before_scale_down))
                if self.idle_time_before_scale_down
                else None
            ),
        )
        remote_login_public_access = "Enabled"
        disableLocalAuth = not self.ssh_settings
        if self.ssh_public_access_enabled is not None:
            remote_login_public_access = "Enabled" if self.ssh_public_access_enabled else "Disabled"

        else:
            remote_login_public_access = "NotSpecified"
        aml_prop = AmlComputeProperties(
            vm_size=self.size if self.size else ComputeDefaults.VMSIZE,
            vm_priority=snake_to_pascal(self.tier),
            user_account_credentials=(self.ssh_settings._to_user_account_credentials() if self.ssh_settings else None),
            scale_settings=scale_settings,
            subnet=subnet_resource,
            remote_login_port_public_access=remote_login_public_access,
            enable_node_public_ip=self.enable_node_public_ip,
        )

        aml_comp = AmlComputeRest(
            description=self.description,
            compute_type=self.type,
            properties=aml_prop,
            disable_local_auth=disableLocalAuth,
        )
        return ComputeResource(
            location=self.location,
            properties=aml_comp,
            identity=(self.identity._to_compute_rest_object() if self.identity else None),
            tags=self.tags,
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

from marshmallow import fields, post_load

from azure.ai.ml._schema.core.fields import NestedField
from azure.ai.ml._schema.core.schema import PatchedSchemaMeta

from .endpoint_connection import EndpointConnectionSchema


class PrivateEndpointSchema(metaclass=PatchedSchemaMeta):
    approval_type = fields.Str()
    connections = fields.Dict(keys=fields.Str(), values=NestedField(EndpointConnectionSchema))

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities import PrivateEndpoint

        return PrivateEndpoint(**data)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Optional, Union

from azure.ai.ml._restclient.v2023_04_01_preview.models import NlpLearningRateScheduler, NlpParameterSubspace
from azure.ai.ml._utils.utils import camel_to_snake
from azure.ai.ml.constants import NlpModels
from azure.ai.ml.entities._job.automl.search_space import SearchSpace
from azure.ai.ml.entities._job.automl.search_space_utils import _convert_from_rest_object, _convert_to_rest_object
from azure.ai.ml.entities._job.sweep.search_space import Choice, SweepDistribution
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class NlpSearchSpace(RestTranslatableMixin):
    """Search space for AutoML NLP tasks.

    :param gradient_accumulation_steps: number of steps over which to accumulate gradients before a backward
        pass. This must be a positive integer., defaults to None
    :type gradient_accumulation_steps: Optional[Union[int, SweepDistribution]]
    :param learning_rate: initial learning rate. Must be a float in (0, 1), defaults to None
    :type learning_rate: Optional[Union[float, SweepDistribution]]
    :param learning_rate_scheduler: the type of learning rate scheduler. Must choose from 'linear', 'cosine',
        'cosine_with_restarts', 'polynomial', 'constant', and 'constant_with_warmup', defaults to None
    :type learning_rate_scheduler: Optional[Union[str, SweepDistribution]]
    :param model_name: the model name to use during training. Must choose from 'bert-base-cased',
        'bert-base-uncased', 'bert-base-multilingual-cased', 'bert-base-german-cased', 'bert-large-cased',
        'bert-large-uncased', 'distilbert-base-cased', 'distilbert-base-uncased', 'roberta-base', 'roberta-large',
        'distilroberta-base', 'xlm-roberta-base', 'xlm-roberta-large', xlnet-base-cased', and 'xlnet-large-cased',
        defaults to None
    :type model_name: Optional[Union[str, SweepDistribution]]
    :param number_of_epochs: the number of epochs to train with. Must be a positive integer, defaults to None
    :type number_of_epochs: Optional[Union[int, SweepDistribution]]
    :param training_batch_size: the batch size during training. Must be a positive integer, defaults to None
    :type training_batch_size: Optional[Union[int, SweepDistribution]]
    :param validation_batch_size: the batch size during validation. Must be a positive integer, defaults to None
    :type validation_batch_size: Optional[Union[int, SweepDistribution]]
    :param warmup_ratio: ratio of total training steps used for a linear warmup from 0 to learning_rate.
            Must be a float in [0, 1], defaults to None
    :type warmup_ratio: Optional[Union[float, SweepDistribution]]
    :param weight_decay: value of weight decay when optimizer is sgd, adam, or adamw. This must be a float in
            the range [0, 1], defaults to None
    :type weight_decay: Optional[Union[float, SweepDistribution]]


    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_automl_nlp.py
                :start-after: [START automl.nlp_search_space]
                :end-before: [END automl.nlp_search_space]
                :language: python
                :dedent: 8
                :caption: creating an nlp search space
    """

    def __init__(
        self,
        *,
        gradient_accumulation_steps: Optional[Union[int, SweepDistribution]] = None,
        learning_rate: Optional[Union[float, SweepDistribution]] = None,
        learning_rate_scheduler: Optional[Union[str, SweepDistribution]] = None,
        model_name: Optional[Union[str, SweepDistribution]] = None,
        number_of_epochs: Optional[Union[int, SweepDistribution]] = None,
        training_batch_size: Optional[Union[int, SweepDistribution]] = None,
        validation_batch_size: Optional[Union[int, SweepDistribution]] = None,
        warmup_ratio: Optional[Union[float, SweepDistribution]] = None,
        weight_decay: Optional[Union[float, SweepDistribution]] = None
    ):
        # Since we want customers to be able to specify enums as well rather than just strings, we need to access
        # the enum values here before we serialize them ('NlpModels.BERT_BASE_CASED' vs. 'bert-base-cased').
        if isinstance(learning_rate_scheduler, NlpLearningRateScheduler):
            learning_rate_scheduler = camel_to_snake(learning_rate_scheduler.value)
        elif isinstance(learning_rate_scheduler, Choice):
            if learning_rate_scheduler.values is not None:
                learning_rate_scheduler.values = [
                    camel_to_snake(item.value) if isinstance(item, NlpLearningRateScheduler) else item
                    for item in learning_rate_scheduler.values
                ]

        if isinstance(model_name, NlpModels):
            model_name = model_name.value
        elif isinstance(model_name, Choice):
            if model_name.values is not None:
                model_name.values = [item.value if isinstance(item, NlpModels) else item for item in model_name.values]

        self.gradient_accumulation_steps = gradient_accumulation_steps
        self.learning_rate = learning_rate
        self.learning_rate_scheduler = learning_rate_scheduler
        self.model_name = model_name
        self.number_of_epochs = number_of_epochs
        self.training_batch_size = training_batch_size
        self.validation_batch_size = validation_batch_size
        self.warmup_ratio = warmup_ratio
        self.weight_decay = weight_decay

    def _to_rest_object(self) -> NlpParameterSubspace:
        return NlpParameterSubspace(
            gradient_accumulation_steps=(
                _convert_to_rest_object(self.gradient_accumulation_steps)
                if self.gradient_accumulation_steps is not None
                else None
            ),
            learning_rate=_convert_to_rest_object(self.learning_rate) if self.learning_rate is not None else None,
            learning_rate_scheduler=(
                _convert_to_rest_object(self.learning_rate_scheduler)
                if self.learning_rate_scheduler is not None
                else None
            ),
            model_name=_convert_to_rest_object(self.model_name) if self.model_name is not None else None,
            number_of_epochs=(
                _convert_to_rest_object(self.number_of_epochs) if self.number_of_epochs is not None else None
            ),
            training_batch_size=(
                _convert_to_rest_object(self.training_batch_size) if self.training_batch_size is not None else None
            ),
            validation_batch_size=(
                _convert_to_rest_object(self.validation_batch_size) if self.validation_batch_size is not None else None
            ),
            warmup_ratio=_convert_to_rest_object(self.warmup_ratio) if self.warmup_ratio is not None else None,
            weight_decay=_convert_to_rest_object(self.weight_decay) if self.weight_decay is not None else None,
        )

    @classmethod
    def _from_rest_object(cls, obj: NlpParameterSubspace) -> "NlpSearchSpace":
        return cls(
            gradient_accumulation_steps=(
                _convert_from_rest_object(obj.gradient_accumulation_steps)
                if obj.gradient_accumulation_steps is not None
                else None
            ),
            learning_rate=_convert_from_rest_object(obj.learning_rate) if obj.learning_rate is not None else None,
            learning_rate_scheduler=(
                _convert_from_rest_object(obj.learning_rate_scheduler)
                if obj.learning_rate_scheduler is not None
                else None
            ),
            model_name=_convert_from_rest_object(obj.model_name) if obj.model_name is not None else None,
            number_of_epochs=(
                _convert_from_rest_object(obj.number_of_epochs) if obj.number_of_epochs is not None else None
            ),
            training_batch_size=(
                _convert_from_rest_object(obj.training_batch_size) if obj.training_batch_size is not None else None
            ),
            validation_batch_size=(
                _convert_from_rest_object(obj.validation_batch_size) if obj.validation_batch_size is not None else None
            ),
            warmup_ratio=_convert_from_rest_object(obj.warmup_ratio) if obj.warmup_ratio is not None else None,
            weight_decay=_convert_from_rest_object(obj.weight_decay) if obj.weight_decay is not None else None,
        )

    @classmethod
    def _from_search_space_object(cls, obj: SearchSpace) -> "NlpSearchSpace":
        return cls(
            gradient_accumulation_steps=(
                obj.gradient_accumulation_steps if hasattr(obj, "gradient_accumulation_steps") else None
            ),
            learning_rate=obj.learning_rate if hasattr(obj, "learning_rate") else None,
            learning_rate_scheduler=obj.learning_rate_scheduler if hasattr(obj, "learning_rate_scheduler") else None,
            model_name=obj.model_name if hasattr(obj, "model_name") else None,
            number_of_epochs=obj.number_of_epochs if hasattr(obj, "number_of_epochs") else None,
            training_batch_size=obj.training_batch_size if hasattr(obj, "training_batch_size") else None,
            validation_batch_size=obj.validation_batch_size if hasattr(obj, "validation_batch_size") else None,
            warmup_ratio=obj.warmup_ratio if hasattr(obj, "warmup_ratio") else None,
            weight_decay=obj.weight_decay if hasattr(obj, "weight_decay") else None,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, NlpSearchSpace):
            return NotImplemented

        return (
            self.gradient_accumulation_steps == other.gradient_accumulation_steps
            and self.learning_rate == other.learning_rate
            and self.learning_rate_scheduler == other.learning_rate_scheduler
            and self.model_name == other.model_name
            and self.number_of_epochs == other.number_of_epochs
            and self.training_batch_size == other.training_batch_size
            and self.validation_batch_size == other.validation_batch_size
            and self.warmup_ratio == other.warmup_ratio
            and self.weight_decay == other.weight_decay
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from datetime import datetime, timedelta
from typing import Any, Dict, Optional

from azure.ai.ml._restclient.v2023_10_01.models import JobBase as RestJobBase
from azure.ai.ml.entities._mixins import RestTranslatableMixin
from azure.ai.ml.entities._system_data import SystemData

from .materialization_type import MaterializationType

FeaturestoreJobTypeMap: Dict[str, MaterializationType] = {
    "BackfillMaterialization": MaterializationType.BACKFILL_MATERIALIZATION,
    "RecurrentMaterialization": MaterializationType.RECURRENT_MATERIALIZATION,
}


class FeatureSetMaterializationMetadata(RestTranslatableMixin):
    """Feature Set Materialization Metadata

    :param type: The type of the materialization job.
    :type type: MaterializationType
    :param feature_window_start_time: The feature window start time for the feature set materialization job.
    :type feature_window_start_time: Optional[datetime]
    :param feature_window_end_time: The feature window end time for the feature set materialization job.
    :type feature_window_end_time: Optional[datetime]
    :param name: The name of the feature set materialization job.
    :type name: Optional[str]
    :param display_name: The display name for the feature set materialization job.
    :type display_name: Optional[str]
    :param creation_context: The creation context of the feature set materialization job.
    :type creation_context: Optional[~azure.ai.ml.entities.SystemData]
    :param duration: current time elapsed for feature set materialization job.
    :type duration: Optional[~datetime.timedelta]
    :param status: The status of the feature set materialization job.
    :type status: Optional[str]
    :param tags: Tag dictionary. Tags can be added, removed, and updated.
    :type tags: Optional[dict[str, str]]
    :param kwargs: A dictionary of additional configuration parameters.
    :type kwargs: dict
    """

    def __init__(
        self,
        *,
        # pylint: disable=redefined-builtin
        type: Optional[MaterializationType],
        feature_window_start_time: Optional[datetime],
        feature_window_end_time: Optional[datetime],
        name: Optional[str],
        display_name: Optional[str],
        creation_context: Optional[SystemData],
        duration: Optional[timedelta],
        status: Optional[str],
        tags: Optional[Dict[str, str]],
        # pylint: disable=unused-argument
        **kwargs: Any,
    ):
        self.type = type
        self.feature_window_start_time = feature_window_start_time
        self.feature_window_end_time = feature_window_end_time
        self.name = name
        self.display_name = display_name
        self.creation_context = creation_context
        self.duration = duration
        self.status = status
        self.tags = tags

    @classmethod
    def _from_rest_object(cls, obj: RestJobBase) -> Optional["FeatureSetMaterializationMetadata"]:
        if not obj:
            return None
        job_properties = obj.properties
        job_type = job_properties.properties.get("azureml.FeatureStoreJobType", None)
        feature_window_start_time = job_properties.properties.get("azureml.FeatureWindowStart", None)
        feature_window_end_time = job_properties.properties.get("azureml.FeatureWindowEnd", None)

        time_format = "%Y-%m-%dT%H:%M:%SZ"
        feature_window_start_time = (
            datetime.strptime(feature_window_start_time, time_format) if feature_window_start_time else None
        )
        feature_window_end_time = (
            datetime.strptime(feature_window_end_time, time_format) if feature_window_end_time else None
        )

        return FeatureSetMaterializationMetadata(
            type=FeaturestoreJobTypeMap.get(job_type),
            feature_window_start_time=feature_window_start_time,
            feature_window_end_time=feature_window_end_time,
            name=obj.name,
            display_name=job_properties.display_name,
            creation_context=SystemData(created_at=obj.system_data.created_at),
            status=job_properties.status,
            tags=job_properties.tags,
            duration=None,
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access

from typing import Dict, Optional

from marshmallow.exceptions import ValidationError as SchemaValidationError

from azure.ai.ml._exception_helper import log_and_raise_error
from azure.ai.ml._restclient.v2023_10_01 import AzureMachineLearningServices as ServiceClient102023
from azure.ai.ml._restclient.v2023_10_01.models import FeaturestoreEntityVersion
from azure.ai.ml._scope_dependent_operations import OperationConfig, OperationScope, _ScopeDependentOperations
from azure.ai.ml._telemetry import ActivityType, monitor_with_activity
from azure.ai.ml._utils._feature_store_utils import _archive_or_restore
from azure.ai.ml._utils._logger_utils import OpsLogger
from azure.ai.ml.constants import ListViewType
from azure.ai.ml.entities._feature_store_entity.feature_store_entity import FeatureStoreEntity
from azure.ai.ml.exceptions import ValidationException
from azure.core.paging import ItemPaged
from azure.core.polling import LROPoller
from azure.core.tracing.decorator import distributed_trace

ops_logger = OpsLogger(__name__)
module_logger = ops_logger.module_logger


class FeatureStoreEntityOperations(_ScopeDependentOperations):
    """FeatureStoreEntityOperations.

    You should not instantiate this class directly. Instead, you should
    create an MLClient instance that instantiates it for you and
    attaches it as an attribute.
    """

    def __init__(
        self,
        operation_scope: OperationScope,
        operation_config: OperationConfig,
        service_client: ServiceClient102023,
        **kwargs: Dict,
    ):
        super(FeatureStoreEntityOperations, self).__init__(operation_scope, operation_config)
        ops_logger.update_filter()
        self._operation = service_client.featurestore_entity_versions
        self._container_operation = service_client.featurestore_entity_containers
        self._service_client = service_client
        self._init_kwargs = kwargs

    @distributed_trace
    @monitor_with_activity(ops_logger, "FeatureStoreEntity.List", ActivityType.PUBLICAPI)
    def list(
        self,
        name: Optional[str] = None,
        *,
        list_view_type: ListViewType = ListViewType.ACTIVE_ONLY,
        **kwargs: Dict,
    ) -> ItemPaged[FeatureStoreEntity]:
        """List the FeatureStoreEntity assets of the workspace.

        :param name: Name of a specific FeatureStoreEntity asset, optional.
        :type name: Optional[str]
        :keyword list_view_type: View type for including/excluding (for example) archived FeatureStoreEntity assets.
            Default: ACTIVE_ONLY.
        :paramtype list_view_type: Optional[ListViewType]
        :return: An iterator like instance of FeatureStoreEntity objects
        :rtype: ~azure.core.paging.ItemPaged[FeatureStoreEntity]
        """
        if name:
            return self._operation.list(
                workspace_name=self._workspace_name,
                name=name,
                cls=lambda objs: [FeatureStoreEntity._from_rest_object(obj) for obj in objs],
                list_view_type=list_view_type,
                **self._scope_kwargs,
                **kwargs,
            )
        return self._container_operation.list(
            workspace_name=self._workspace_name,
            cls=lambda objs: [FeatureStoreEntity._from_container_rest_object(obj) for obj in objs],
            list_view_type=list_view_type,
            **self._scope_kwargs,
            **kwargs,
        )

    def _get(self, name: str, version: Optional[str] = None, **kwargs: Dict) -> FeaturestoreEntityVersion:
        return self._operation.get(
            resource_group_name=self._resource_group_name,
            workspace_name=self._workspace_name,
            name=name,
            version=version,
            **self._init_kwargs,
            **kwargs,
        )

    @distributed_trace
    @monitor_with_activity(ops_logger, "FeatureStoreEntity.Get", ActivityType.PUBLICAPI)
    def get(self, name: str, version: str, **kwargs: Dict) -> FeatureStoreEntity:  # type: ignore
        """Get the specified FeatureStoreEntity asset.

        :param name: Name of FeatureStoreEntity asset.
        :type name: str
        :param version: Version of FeatureStoreEntity asset.
        :type version: str
        :raises ~azure.ai.ml.exceptions.ValidationException: Raised if FeatureStoreEntity cannot be successfully
            identified and retrieved. Details will be provided in the error message.
        :return: FeatureStoreEntity asset object.
        :rtype: ~azure.ai.ml.entities.FeatureStoreEntity
        """
        try:
            feature_store_entity_version_resource = self._get(name, version, **kwargs)
            return FeatureStoreEntity._from_rest_object(feature_store_entity_version_resource)
        except (ValidationException, SchemaValidationError) as ex:
            log_and_raise_error(ex)

    @distributed_trace
    @monitor_with_activity(ops_logger, "FeatureStoreEntity.BeginCreateOrUpdate", ActivityType.PUBLICAPI)
    def begin_create_or_update(
        self, feature_store_entity: FeatureStoreEntity, **kwargs: Dict
    ) -> LROPoller[FeatureStoreEntity]:
        """Create or update FeatureStoreEntity

        :param feature_store_entity: FeatureStoreEntity definition.
        :type feature_store_entity: FeatureStoreEntity
        :return: An instance of LROPoller that returns a FeatureStoreEntity.
        :rtype: ~azure.core.polling.LROPoller[~azure.ai.ml.entities.FeatureStoreEntity]
        """
        feature_store_entity_resource = FeatureStoreEntity._to_rest_object(feature_store_entity)

        return self._operation.begin_create_or_update(
            resource_group_name=self._resource_group_name,
            workspace_name=self._workspace_name,
            name=feature_store_entity.name,
            version=feature_store_entity.version,
            body=feature_store_entity_resource,
            cls=lambda response, deserialized, headers: FeatureStoreEntity._from_rest_object(deserialized),
            **kwargs,
        )

    @distributed_trace
    @monitor_with_activity(ops_logger, "FeatureStoreEntity.Archive", ActivityType.PUBLICAPI)
    def archive(
        self,
        name: str,
        version: str,
        **kwargs: Dict,
    ) -> None:
        """Archive a FeatureStoreEntity asset.

        :param name: Name of FeatureStoreEntity asset.
        :type name: str
        :param version: Version of FeatureStoreEntity asset.
        :type version: str
        :return: None
        """

        _archive_or_restore(
            asset_operations=self,
            version_operation=self._operation,
            is_archived=True,
            name=name,
            version=version,
            **kwargs,
        )

    @distributed_trace
    @monitor_with_activity(ops_logger, "FeatureStoreEntity.Restore", ActivityType.PUBLICAPI)
    def restore(
        self,
        name: str,
        version: str,
        **kwargs: Dict,
    ) -> None:
        """Restore an archived FeatureStoreEntity asset.

        :param name: Name of FeatureStoreEntity asset.
        :type name: str
        :param version: Version of FeatureStoreEntity asset.
        :type version: str
        :return: None
        """

        _archive_or_restore(
            asset_operations=self,
            version_operation=self._operation,
            is_archived=False,
            name=name,
            version=version,
            **kwargs,
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

"""Contains supported operations for Azure Machine Learning SDKv2.

Operations are classes contain logic to interact with backend services, usually auto generated operations call.
"""
__path__ = __import__("pkgutil").extend_path(__path__, __name__)


from ._azure_openai_deployment_operations import AzureOpenAIDeploymentOperations
from ._batch_deployment_operations import BatchDeploymentOperations
from ._batch_endpoint_operations import BatchEndpointOperations
from ._component_operations import ComponentOperations
from ._compute_operations import ComputeOperations
from ._data_operations import DataOperations
from ._datastore_operations import DatastoreOperations
from ._environment_operations import EnvironmentOperations
from ._feature_set_operations import FeatureSetOperations
from ._feature_store_entity_operations import FeatureStoreEntityOperations
from ._feature_store_operations import FeatureStoreOperations
from ._index_operations import IndexOperations
from ._job_operations import JobOperations
from ._model_operations import ModelOperations
from ._online_deployment_operations import OnlineDeploymentOperations
from ._online_endpoint_operations import OnlineEndpointOperations
from ._registry_operations import RegistryOperations
from ._schedule_operations import ScheduleOperations
from ._workspace_connections_operations import WorkspaceConnectionsOperations
from ._workspace_operations import WorkspaceOperations
from ._workspace_outbound_rule_operations import WorkspaceOutboundRuleOperations
from ._evaluator_operations import EvaluatorOperations
from ._serverless_endpoint_operations import ServerlessEndpointOperations
from ._marketplace_subscription_operations import MarketplaceSubscriptionOperations
from ._capability_hosts_operations import CapabilityHostsOperations

__all__ = [
    "ComputeOperations",
    "DatastoreOperations",
    "JobOperations",
    "ModelOperations",
    "EvaluatorOperations",
    "WorkspaceOperations",
    "RegistryOperations",
    "OnlineEndpointOperations",
    "BatchEndpointOperations",
    "OnlineDeploymentOperations",
    "BatchDeploymentOperations",
    "DataOperations",
    "EnvironmentOperations",
    "ComponentOperations",
    "WorkspaceConnectionsOperations",
    "RegistryOperations",
    "ScheduleOperations",
    "WorkspaceOutboundRuleOperations",
    "FeatureSetOperations",
    "FeatureStoreEntityOperations",
    "FeatureStoreOperations",
    "ServerlessEndpointOperations",
    "MarketplaceSubscriptionOperations",
    "IndexOperations",
    "AzureOpenAIDeploymentOperations",
    "CapabilityHostsOperations",
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from marshmallow import fields

from azure.ai.ml._schema.core.fields import NestedField, StringTransformedEnum
from azure.ai.ml._schema.job.input_output_fields_provider import InputsField, OutputsField
from azure.ai.ml.constants import JobType

from .base_job import BaseJobSchema
from .job_limits import CommandJobLimitsSchema
from .parameterized_command import ParameterizedCommandSchema


class CommandJobSchema(ParameterizedCommandSchema, BaseJobSchema):
    type = StringTransformedEnum(allowed_values=JobType.COMMAND)
    # do not promote it as CommandComponent has no field named 'limits'
    limits = NestedField(CommandJobLimitsSchema)
    parameters = fields.Dict(dump_only=True)
    inputs = InputsField()
    outputs = OutputsField()
    parent_job_name = fields.Str()



import contextlib
import dataclasses
import os
import shutil
import tempfile
from pathlib import Path
from typing import Callable, List, Tuple

import pytest

from azure.ai.ml._utils._asset_utils import (
    AmlIgnoreFile,
    GitIgnoreFile,
    IgnoreFile,
    _check_or_modify_auto_delete_setting,
    _validate_auto_delete_setting_in_data_output,
    _validate_workspace_managed_datastore,
    get_directory_size,
    get_ignore_file,
    get_object_hash,
    get_upload_files_from_folder,
)
from azure.ai.ml._utils.utils import convert_windows_path_to_unix
from azure.ai.ml.constants._common import AutoDeleteCondition
from azure.ai.ml.entities._assets.auto_delete_setting import AutoDeleteSetting
from azure.ai.ml.exceptions import AssetPathException, ValidationException


@pytest.fixture
def storage_test_directory() -> str:
    with tempfile.TemporaryDirectory() as temp_dir:
        shutil.rmtree(temp_dir)
        shutil.copytree("./tests/test_configs/storage/", temp_dir)
        yield temp_dir


@pytest.fixture
def gitignore_file_directory(storage_test_directory: str) -> str:
    return os.path.join(storage_test_directory, "gitignore_only")


@pytest.fixture
def no_ignore_file_directory(storage_test_directory: str) -> str:
    return os.path.join(storage_test_directory, "dont_include_us")


@pytest.fixture
def amlignore_file(storage_test_directory: str) -> AmlIgnoreFile:
    return AmlIgnoreFile(storage_test_directory)


@pytest.fixture
def gitignore_file(storage_test_directory: str) -> GitIgnoreFile:
    return GitIgnoreFile(storage_test_directory)


@pytest.fixture
def no_ignore_file(no_ignore_file_directory: str) -> IgnoreFile:
    return IgnoreFile(None)


def generate_link_file(base_dir: str) -> Tuple[os.PathLike, os.PathLike]:
    target_file_name = "target_file_rand_name.txt"
    target_file = Path(os.path.join(os.path.abspath(base_dir), target_file_name))
    target_file.write_text("some text")

    link_file_name = "link_file_rand_name.txt"
    link_file = Path(os.path.join(os.path.abspath(base_dir), link_file_name))

    os.symlink(target_file, link_file)

    assert os.path.islink(link_file)
    return convert_windows_path_to_unix(target_file), convert_windows_path_to_unix(link_file)


@dataclasses.dataclass
class SymbolLinkTestCase:
    target_folder: str
    link_folder: str
    temp_folder: str
    target_file: str = None
    expected_remote_paths_without_prefix: List[str] = None

    @classmethod
    @contextlib.contextmanager
    def _base(cls) -> "SymbolLinkTestCase":
        with tempfile.TemporaryDirectory() as base_dir:
            test_case = cls(
                target_folder=os.path.join(base_dir, "target_folder"),
                link_folder=os.path.join(base_dir, "link_folder"),
                temp_folder=os.path.join(base_dir, "temp_folder"),
            )
            os.mkdir(test_case.target_folder)
            os.mkdir(test_case.link_folder)
            os.mkdir(test_case.temp_folder)

            test_case.target_file = os.path.join(test_case.target_folder, "target_file.txt")
            with open(test_case.target_file, "w") as f:
                f.write("file content")
            yield test_case

    @classmethod
    @contextlib.contextmanager
    def intermediate_link(cls) -> "SymbolLinkTestCase":
        with cls._base() as test_case:
            Path(test_case.temp_folder, "intermediate_file.txt").symlink_to(test_case.target_file)
            link_file = os.path.join(test_case.link_folder, "link_file.txt")
            Path(link_file).symlink_to(os.path.join(test_case.temp_folder, "intermediate_file.txt"))
            test_case.expected_remote_paths_without_prefix = [
                "link_file.txt",
            ]
            yield test_case

    @classmethod
    @contextlib.contextmanager
    def relative_file(cls) -> "SymbolLinkTestCase":
        with cls._base() as test_case:
            link_file = os.path.join(test_case.link_folder, "link_file.txt")
            cur_dir = os.getcwd()
            os.chdir(test_case.link_folder)
            os.symlink(src=os.path.relpath(test_case.target_file, test_case.link_folder), dst=link_file)
            os.chdir(cur_dir)
            test_case.expected_remote_paths_without_prefix = [
                "link_file.txt",
            ]
            yield test_case

    @classmethod
    @contextlib.contextmanager
    def relative_folder(cls) -> "SymbolLinkTestCase":
        with cls._base() as test_case:
            link_path = os.path.join(test_case.link_folder, "link_folder")
            cur_dir = os.getcwd()
            os.chdir(test_case.link_folder)
            os.symlink(src=os.path.relpath(test_case.target_folder, test_case.link_folder), dst=link_path)
            os.chdir(cur_dir)
            test_case.expected_remote_paths_without_prefix = [
                "link_folder/target_file.txt",
            ]
            yield test_case

    @classmethod
    @contextlib.contextmanager
    def relative_intermediate_folder(cls) -> "SymbolLinkTestCase":
        with cls._base() as test_case:
            cur_dir = os.getcwd()
            temp_base = os.path.join(test_case.temp_folder, "sub_folder")
            os.mkdir(temp_base)
            inter_folder = os.path.join(temp_base, "inter")
            os.chdir(temp_base)
            os.symlink(src=os.path.relpath(test_case.target_folder, temp_base), dst=inter_folder)

            link_path = os.path.join(test_case.link_folder, "link_folder")
            os.chdir(test_case.link_folder)
            os.symlink(src=os.path.relpath(temp_base, test_case.link_folder), dst=link_path)
            os.chdir(cur_dir)
            test_case.expected_remote_paths_without_prefix = [
                "link_folder/inter/target_file.txt",
            ]
            yield test_case

    @classmethod
    @contextlib.contextmanager
    def basic_file(cls) -> "SymbolLinkTestCase":
        with cls._base() as test_case:
            link_file = os.path.join(test_case.link_folder, "link_file.txt")
            os.symlink(src=test_case.target_file, dst=link_file)
            test_case.expected_remote_paths_without_prefix = [
                "link_file.txt",
            ]
            yield test_case


@pytest.mark.unittest
@pytest.mark.core_sdk_test
class TestAssetUtils:
    def test_amlignore_precedence(
        self, storage_test_directory: str, gitignore_file_directory: str, no_ignore_file_directory: str
    ) -> None:
        amlignore_file = get_ignore_file(directory_path=storage_test_directory)
        assert isinstance(amlignore_file, AmlIgnoreFile)

        gitignore_file = get_ignore_file(directory_path=gitignore_file_directory)
        assert isinstance(gitignore_file, GitIgnoreFile)

        no_ignore_file = get_ignore_file(directory_path=no_ignore_file_directory)
        assert isinstance(no_ignore_file, IgnoreFile)
        assert no_ignore_file.path is None

    def test_hashing(
        self,
        storage_test_directory: str,
        amlignore_file: AmlIgnoreFile,
        gitignore_file: GitIgnoreFile,
        no_ignore_file: IgnoreFile,
    ) -> None:
        no_ignore_hash = get_object_hash(path=storage_test_directory, ignore_file=no_ignore_file)
        amlignore_hash = get_object_hash(path=storage_test_directory, ignore_file=amlignore_file)
        gitignore_hash = get_object_hash(path=storage_test_directory, ignore_file=gitignore_file)

        assert no_ignore_hash != amlignore_hash != gitignore_hash

    def test_exclude(
        self,
        storage_test_directory: str,
        amlignore_file: AmlIgnoreFile,
        gitignore_file: GitIgnoreFile,
        no_ignore_file: IgnoreFile,
    ) -> None:
        source_path = Path(storage_test_directory).resolve()
        prefix = source_path.name + "/"

        amlignore_upload_paths = get_upload_files_from_folder(
            source_path,
            prefix=prefix,
            ignore_file=amlignore_file,
        )
        gitignore_upload_paths = get_upload_files_from_folder(
            source_path,
            prefix=prefix,
            ignore_file=gitignore_file,
        )
        no_ignore_upload_paths = get_upload_files_from_folder(
            source_path,
            prefix=prefix,
            ignore_file=no_ignore_file,
        )

        assert len(no_ignore_upload_paths) == 7
        assert len(gitignore_upload_paths) == 6
        assert len(amlignore_upload_paths) == 4

    def test_upload_paths_match(self, storage_test_directory: str) -> None:
        source_path = Path(storage_test_directory).resolve()
        prefix = source_path.name + "/"
        upload_paths = get_upload_files_from_folder(
            source_path,
            prefix=prefix,
        )

        for local_path, remote_path in upload_paths:
            remote_path = remote_path.split("/", 2)[-1]  # strip LocalUpload/<asset id> prefix
            if remote_path.startswith("link_file_"):  # ignore symlinks because their remote and local paths will differ
                continue
            assert remote_path in local_path

    @pytest.mark.parametrize(
        "create_test_case",
        [
            pytest.param(SymbolLinkTestCase.basic_file, id="basic_file"),
            pytest.param(SymbolLinkTestCase.intermediate_link, id="intermediate_link"),
            pytest.param(SymbolLinkTestCase.relative_file, id="relative_file"),
            pytest.param(SymbolLinkTestCase.relative_folder, id="relative_folder"),
            pytest.param(SymbolLinkTestCase.relative_intermediate_folder, id="relative_intermediate_folder"),
        ],
    )
    def test_symlinks_included_in_hash(
        self, create_test_case: Callable[[], SymbolLinkTestCase], no_ignore_file: IgnoreFile
    ) -> None:
        """Confirm that changes in the original file are respected when the symlink is hashed"""
        with create_test_case() as test_case:
            # hash symlink, update original file, hash symlink again and compare hashes
            original_hash = get_object_hash(path=test_case.link_folder, ignore_file=no_ignore_file)
            Path(test_case.target_file).write_text("some more text")
            updated_hash = get_object_hash(path=test_case.link_folder, ignore_file=no_ignore_file)
            assert original_hash != updated_hash

    @pytest.mark.parametrize(
        "create_test_case",
        [
            pytest.param(SymbolLinkTestCase.basic_file, id="basic_file"),
            pytest.param(SymbolLinkTestCase.intermediate_link, id="intermediate_link"),
            pytest.param(SymbolLinkTestCase.relative_file, id="relative_file"),
            pytest.param(SymbolLinkTestCase.relative_folder, id="relative_folder"),
            pytest.param(SymbolLinkTestCase.relative_intermediate_folder, id="relative_intermediate_folder"),
        ],
    )
    def test_symlink_upload_paths(self, create_test_case: Callable[[], SymbolLinkTestCase]) -> None:
        """Confirm that symlink name is preserved for upload to storage, but that target file's path is uploaded

        e.g given a file ./dir/foo/bar.txt with a symlink ./other_dir/bar_link.txt, we want to upload the contents of ./dir/food/bar.txt at path ./other_dir/bar_link.txt in the remote storage.
        """
        prefix = "random_prefix/"
        with create_test_case() as test_case:
            upload_paths_list = get_upload_files_from_folder(
                test_case.link_folder,
                prefix=prefix,
            )

            local_paths = [i for i, _ in upload_paths_list]
            remote_paths = [j for _, j in upload_paths_list]

            # When username is too long, temp folder path will be truncated, e.g. longusername -> LONGUS~
            # so resolve target_file_path to get the full path
            assert Path(test_case.target_file).resolve().as_posix() in local_paths
            # remote file names are relative to root and include the prefix
            assert list(sorted(remote_paths)) == list(
                sorted([prefix + p for p in test_case.expected_remote_paths_without_prefix])
            )

    def test_directory_size_with_ignore_file(self, storage_test_directory: str, amlignore_file: AmlIgnoreFile) -> None:
        base_size = get_directory_size(storage_test_directory)
        with_ignore_size = get_directory_size(storage_test_directory, ignore_file=amlignore_file)

        # Note, the [1] index is the number of files counted in the directory size calculation.
        # The [0] index is the sum file size, which we don't check here due to how instable that
        # value is across systems/builds.
        # Directory size calculated with ignore file should include less files
        assert len(with_ignore_size[1]) < len(base_size[1])

        # Directory size calculated after symlink creation should correctly include linked file size,
        # and count symlink file itself towards file count.
        _, _ = generate_link_file(storage_test_directory)
        with_symlink_size = get_directory_size(storage_test_directory)
        assert len(with_symlink_size[1]) == len(base_size[1]) + 2

    def test_check_or_modify_auto_delete_setting(self):
        _check_or_modify_auto_delete_setting(None)

        setting = {"condition": "created_greater_than"}
        _check_or_modify_auto_delete_setting(setting)
        assert setting["condition"] == "createdGreaterThan"

        setting = AutoDeleteSetting(condition=AutoDeleteCondition.CREATED_GREATER_THAN)
        _check_or_modify_auto_delete_setting(setting)
        assert setting.condition == "createdGreaterThan"

    def test_validate_auto_delete_setting_in_data_output(self):
        with pytest.raises(ValidationException):
            _validate_auto_delete_setting_in_data_output({"value": "30d"})

        _validate_auto_delete_setting_in_data_output(None)
        _validate_auto_delete_setting_in_data_output({})

    def test_validate_workspace_managed_datastore(self):
        with pytest.raises(AssetPathException):
            _validate_workspace_managed_datastore("azureml://datastores/workspacemanageddatastore/123")

        path = _validate_workspace_managed_datastore("azureml://datastores/workspacemanageddatastore")
        assert path == "azureml://datastores/workspacemanageddatastore/paths"

        path = _validate_workspace_managed_datastore("azureml://datastores/workspacemanageddatastore/")
        assert path == "azureml://datastores/workspacemanageddatastore/paths"

        path = _validate_workspace_managed_datastore("azureml://datastores/workspacemanageddatastore123")
        assert path == "azureml://datastores/workspacemanageddatastore123"



from datetime import datetime, timedelta

_SCHEDULE_TIMEOUT_SECOND = 20 * 60  # timeout for schedule's tests, unit in second.

TRIGGER_ENDTIME = (datetime(year=2099, month=1, day=1)).strftime("%Y-%m-%d %H:%M:%S")
TRIGGER_ENDTIME_DICT = [{"trigger.end_time": TRIGGER_ENDTIME}]  # trigger endtime can not be past time.



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

from os import PathLike
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple, Union

from azure.ai.ml._schema._feature_set.feature_set_backfill_schema import FeatureSetBackfillSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, PARAMS_OVERRIDE_KEY
from azure.ai.ml.entities._feature_set.feature_window import FeatureWindow
from azure.ai.ml.entities._feature_set.materialization_compute_resource import MaterializationComputeResource
from azure.ai.ml.entities._mixins import RestTranslatableMixin
from azure.ai.ml.entities._util import load_from_dict


class FeatureSetBackfillRequest(RestTranslatableMixin):
    """Feature Set Backfill Request

    :param name: The name of the backfill job request
    :type name: str
    :param version: The version of the backfill job request.
    :type version: str
    :param feature_window: The time window for the feature set backfill request.
    :type feature_window: ~azure.ai.ml._restclient.v2023_04_01_preview.models.FeatureWindow
    :param description: The description of the backfill job request. Defaults to None.
    :type description: Optional[str]
    :param tags: Tag dictionary. Tags can be added, removed, and updated. Defaults to None.
    :type tags: Optional[dict[str, str]]
    :keyword resource: The compute resource settings. Defaults to None.
    :paramtype resource: Optional[~azure.ai.ml.entities.MaterializationComputeResource]
    :param spark_configuration: Specifies the spark configuration. Defaults to None.
    :type spark_configuration: Optional[dict[str, str]]
    """

    def __init__(
        self,
        *,
        name: str,
        version: str,
        feature_window: Optional[FeatureWindow] = None,
        description: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
        resource: Optional[MaterializationComputeResource] = None,
        spark_configuration: Optional[Dict[str, str]] = None,
        data_status: Optional[List[str]] = None,
        job_id: Optional[str] = None,
        **kwargs: Any,
    ):
        self.name = name
        self.version = version
        self.feature_window = feature_window
        self.description = description
        self.resource = resource
        self.tags = tags
        self.spark_configuration = spark_configuration
        self.data_status = data_status
        self.job_id = job_id

    @classmethod
    # pylint: disable=unused-argument
    def _resolve_cls_and_type(cls, data: Dict, params_override: Tuple) -> Tuple:
        """Resolve the class to use for deserializing the data. Return current class if no override is provided.

        :param data: Data to deserialize.
        :type data: dict
        :param params_override: Parameters to override, defaults to None
        :type params_override: typing.Optional[list]
        :return: Class to use for deserializing the data & its "type". Type will be None if no override is provided.
        :rtype: tuple[class, typing.Optional[str]]
        """
        return cls, None

    @classmethod
    def _load(
        cls,
        data: Optional[Dict] = None,
        yaml_path: Optional[Union[PathLike, str]] = None,
        params_override: Optional[list] = None,
        **kwargs: Any,
    ) -> "FeatureSetBackfillRequest":
        data = data or {}
        params_override = params_override or []
        context = {
            BASE_PATH_CONTEXT_KEY: Path(yaml_path).parent if yaml_path else Path("./"),
            PARAMS_OVERRIDE_KEY: params_override,
        }
        loaded_schema = load_from_dict(FeatureSetBackfillSchema, data, context, **kwargs)
        return FeatureSetBackfillRequest(**loaded_schema)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import logging
from collections import OrderedDict
from datetime import datetime, timedelta
from html import escape

SUPPORTED_VALUE_TYPE_TUPLE = (int, float, str, datetime, timedelta)
TABLE_FMT = '<table style="width:100%">{0}</table>'
ROW_FMT = "<tr>{0}</tr>"
HEADER_FMT = "<th>{0}</th>"
DATA_FMT = "<td>{0}</td>"
# target="_blank" opens in new tab, rel="noopener" is for perf + security
LINK_FMT = '<a href="{0}" target="_blank" rel="noopener">{1}</a>'


def convert_dict_to_table(object_to_convert):
    # Case 1: All non-collection values -> table of 1 row
    # Case 2: All collection values, lens eq N -> assert lengths eq, table of N rows
    # Case 3: collection and non-collection values -> table of 1 row, collections nest
    # Case 4: All collection values, lens unequal -> table of 1 row, each is nested collection?

    if not isinstance(object_to_convert, dict):
        raise AssertionError("Expected a dict or subclass, got {0}".format(type(object_to_convert)))

    if len(object_to_convert) == 0:
        return ""

    ordered_obj = OrderedDict(object_to_convert)

    def is_collection_type(val):
        return hasattr(val, "__len__") and not isinstance(val, str)

    is_collection = [is_collection_type(value) for value in ordered_obj.values()]

    all_rows = [get_header_row_string(ordered_obj.keys())]

    def values_to_data_row(values):
        cells = [to_html(value) for value in values]
        return get_data_row_string(cells)

    if all(is_collection):
        # Cases 2 and 4
        length = len(list(ordered_obj.values)[0])
        if any(len(v) != length for v in ordered_obj.values()):
            # Case 4
            logging.warning("Uneven column lengths in table conversion")
            all_rows.append(values_to_data_row(ordered_obj.values()))

        else:
            # Case 2 - sad transpose
            for i in range(length):
                value_list = [val[i] for val in ordered_obj.values()]
                all_rows.append(values_to_data_row(value_list))

    else:
        # Cases 1 and 3
        # Table of 1 row of values or mixed types, table of 1 row
        all_rows.append(values_to_data_row(ordered_obj.values()))

    return table_from_html_rows(all_rows)


def convert_list_to_table(object_to_convert):
    if not isinstance(object_to_convert, list):
        raise AssertionError("Expected a list or subclass, got {0}".format(type(object_to_convert)))

    if len(object_to_convert) == 0:
        return ""

    all_values = [to_html(element) for element in object_to_convert]
    all_rows = [get_data_row_string([element]) for element in all_values]
    return table_from_html_rows(all_rows)


# Mapping from complex type to HTML converters
# Unspecified types default to convert_value
_type_to_converter = {list: convert_list_to_table, dict: convert_dict_to_table}


def to_html(object_to_convert):
    candidate_converters = [k for k in _type_to_converter if isinstance(object_to_convert, k)]

    if len(candidate_converters) == 0:
        converter = convert_value
    elif len(candidate_converters) == 1:
        converter = _type_to_converter[candidate_converters[0]]
    else:
        logging.warning("Multiple candidate converters found for type %s", type(object_to_convert))
        converter = convert_value

    converted_value = converter(object_to_convert)
    return converted_value


def is_string_link(string: str) -> bool:
    return isinstance(string, str) and string.strip().lower().startswith("http")


def make_link(link_string: str, link_text: str = "") -> str:
    if not link_text:  # Actually want truthy string
        link_text = "Link"
    return LINK_FMT.format(escape(link_string), link_text)


def convert_value(value: str) -> str:
    if value is None:
        return ""
    if is_string_link(value):
        return make_link(value)
    if not isinstance(value, SUPPORTED_VALUE_TYPE_TUPLE):
        logging.warning("Unsupported type %s for html, converting", type(value))

    # TODO: Figure out a good escaping story here right now it breaks existing tags
    return str(value)


def get_header_row_string(column_headers):
    headers = [HEADER_FMT.format(header) for header in column_headers]
    return ROW_FMT.format("".join(headers))


def get_data_row_string(data_values):
    data = [DATA_FMT.format(datum) for datum in data_values]
    return ROW_FMT.format("".join(data))


def table_from_html_rows(list_of_rows):
    # type/: (List[str]) -> str
    return TABLE_FMT.format("".join(list_of_rows))


def to_formatted_html_table(rows, header):
    html = ["""<table style="width:100%; border:2px solid black" >"""]
    if header is not None:
        html_row = "</td><td>".join(column for column in header)
        html.append(
            """<tr style="font-weight:bold; border-bottom:1pt solid black; border-right: 1pt solid black;
                    text-align: center"><td>{}</td></tr>""".format(
                html_row
            )
        )

    for row in rows:
        html_row = "</td><td>".join(str(value) for value in row)
        html.append(
            """<tr style="width:100%; word-wrap: break-word; border-bottom:1pt solid black;
                    text-align: center"><td>{}</td></tr>""".format(
                html_row
            )
        )
    html.append("</table>")
    return "".join(html)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    NlpVerticalFeaturizationSettings as RestNlpVerticalFeaturizationSettings,
)
from azure.ai.ml.entities._job.automl.featurization_settings import FeaturizationSettings, FeaturizationSettingsType


class NlpFeaturizationSettings(FeaturizationSettings):
    """Featurization settings for all AutoML NLP Verticals.

    :ivar type: Specifies the type of FeaturizationSettings. Set automatically to "NLP" for this class.
    :vartype type: str

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_automl_nlp.py
                :start-after: [START automl.nlp_featurization_settings]
                :end-before: [END automl.nlp_featurization_settings]
                :language: python
                :dedent: 8
                :caption: creating an nlp featurization settings
    """

    type = FeaturizationSettingsType.NLP

    def _to_rest_object(self) -> RestNlpVerticalFeaturizationSettings:
        return RestNlpVerticalFeaturizationSettings(
            dataset_language=self.dataset_language,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestNlpVerticalFeaturizationSettings) -> "NlpFeaturizationSettings":
        return cls(
            dataset_language=obj.dataset_language,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, NlpFeaturizationSettings):
            return NotImplemented

        return super().__eq__(other)

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from datetime import datetime
from typing import Any, Optional

from azure.ai.ml._restclient.v2023_10_01.models import FeatureWindow as RestFeatureWindow
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class FeatureWindow(RestTranslatableMixin):
    """Feature window
    :keyword feature_window_end: Specifies the feature window end time.
    :paramtype feature_window_end: ~datetime.datetime
    :keyword feature_window_start: Specifies the feature window start time.
    :paramtype feature_window_start: ~datetime.datetime
    """

    # pylint: disable=unused-argument
    def __init__(self, *, feature_window_start: datetime, feature_window_end: datetime, **kwargs: Any) -> None:
        self.feature_window_start = feature_window_start
        self.feature_window_end = feature_window_end

    def _to_rest_object(self) -> RestFeatureWindow:
        return RestFeatureWindow(
            feature_window_start=self.feature_window_start, feature_window_end=self.feature_window_end
        )

    @classmethod
    def _from_rest_object(cls, obj: RestFeatureWindow) -> Optional["FeatureWindow"]:
        if not obj:
            return None
        return FeatureWindow(feature_window_start=obj.feature_window_start, feature_window_end=obj.feature_window_end)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access
from typing import Any, Callable, Dict, List, Mapping, Optional, Union, cast

from marshmallow import INCLUDE

from azure.ai.ml import Output
from azure.ai.ml._schema import NestedField
from azure.ai.ml._schema.pipeline.component_job import SweepSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, SOURCE_PATH_CONTEXT_KEY, CommonYamlFields
from azure.ai.ml.constants._component import ControlFlowType, DataTransferTaskType, NodeType
from azure.ai.ml.constants._compute import ComputeType
from azure.ai.ml.dsl._component_func import to_component_func
from azure.ai.ml.dsl._overrides_definition import OverrideDefinition
from azure.ai.ml.entities._builders import (
    BaseNode,
    Command,
    DataTransferCopy,
    DataTransferExport,
    DataTransferImport,
    Import,
    Parallel,
    Spark,
    Sweep,
)
from azure.ai.ml.entities._builders.condition_node import ConditionNode
from azure.ai.ml.entities._builders.control_flow_node import ControlFlowNode
from azure.ai.ml.entities._builders.do_while import DoWhile
from azure.ai.ml.entities._builders.parallel_for import ParallelFor
from azure.ai.ml.entities._builders.pipeline import Pipeline
from azure.ai.ml.entities._component.component import Component
from azure.ai.ml.entities._job.automl.automl_job import AutoMLJob
from azure.ai.ml.entities._util import get_type_from_spec
from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationException


class _PipelineNodeFactory:
    """A class to create pipeline node instances from yaml dict or rest objects without hard-coded type check."""

    def __init__(self) -> None:
        self._create_instance_funcs: dict = {}
        self._load_from_rest_object_funcs: dict = {}

        self.register_type(
            _type=NodeType.COMMAND,
            create_instance_func=lambda: Command.__new__(Command),
            load_from_rest_object_func=Command._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=NodeType.IMPORT,
            create_instance_func=lambda: Import.__new__(Import),
            load_from_rest_object_func=Import._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=NodeType.PARALLEL,
            create_instance_func=lambda: Parallel.__new__(Parallel),
            load_from_rest_object_func=Parallel._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=NodeType.PIPELINE,
            create_instance_func=lambda: Pipeline.__new__(Pipeline),
            load_from_rest_object_func=Pipeline._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=NodeType.SWEEP,
            create_instance_func=lambda: Sweep.__new__(Sweep),
            load_from_rest_object_func=Sweep._from_rest_object,
            nested_schema=NestedField(SweepSchema, unknown=INCLUDE),
        )
        self.register_type(
            _type=NodeType.AUTOML,
            create_instance_func=None,
            load_from_rest_object_func=self._automl_from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=NodeType.SPARK,
            create_instance_func=lambda: Spark.__new__(Spark),
            load_from_rest_object_func=Spark._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=ControlFlowType.DO_WHILE,
            create_instance_func=None,
            load_from_rest_object_func=DoWhile._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=ControlFlowType.IF_ELSE,
            create_instance_func=None,
            load_from_rest_object_func=ConditionNode._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=ControlFlowType.PARALLEL_FOR,
            create_instance_func=None,
            load_from_rest_object_func=ParallelFor._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type="_".join([NodeType.DATA_TRANSFER, DataTransferTaskType.COPY_DATA]),
            create_instance_func=lambda: DataTransferCopy.__new__(DataTransferCopy),
            load_from_rest_object_func=DataTransferCopy._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type="_".join([NodeType.DATA_TRANSFER, DataTransferTaskType.IMPORT_DATA]),
            create_instance_func=lambda: DataTransferImport.__new__(DataTransferImport),
            load_from_rest_object_func=DataTransferImport._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type="_".join([NodeType.DATA_TRANSFER, DataTransferTaskType.EXPORT_DATA]),
            create_instance_func=lambda: DataTransferExport.__new__(DataTransferExport),
            load_from_rest_object_func=DataTransferExport._from_rest_object,
            nested_schema=None,
        )
        self.register_type(
            _type=NodeType.FLOW_PARALLEL,
            create_instance_func=lambda: Parallel.__new__(Parallel),
            load_from_rest_object_func=None,
            nested_schema=None,
        )

    @classmethod
    def _get_func(cls, _type: str, funcs: Dict[str, Callable]) -> Callable:
        if _type == NodeType._CONTAINER:
            msg = (
                "Component returned by 'list' is abbreviated and can not be used directly, "
                "please use result from 'get'."
            )
            raise ValidationException(
                message=msg,
                no_personal_data_message=msg,
                target=ErrorTarget.COMPONENT,
                error_category=ErrorCategory.USER_ERROR,
            )
        _type = get_type_from_spec({CommonYamlFields.TYPE: _type}, valid_keys=funcs)
        return funcs[_type]

    def get_create_instance_func(self, _type: str) -> Callable[..., BaseNode]:
        """Get the function to create a new instance of the node.

        :param _type: The type of the node.
        :type _type: str
        :return: The create instance function
        :rtype: Callable[..., BaseNode]
        """
        return self._get_func(_type, self._create_instance_funcs)

    def get_load_from_rest_object_func(self, _type: str) -> Callable:
        """Get the function to load a node from a rest object.

        :param _type: The type of the node.
        :type _type: str
        :return: The `_load_from_rest_object` function
        :rtype: Callable[[Any], Union[BaseNode, AutoMLJob, ControlFlowNode]]
        """
        return self._get_func(_type, self._load_from_rest_object_funcs)

    def register_type(
        self,
        _type: str,
        *,
        create_instance_func: Optional[Callable[..., Union[BaseNode, AutoMLJob]]] = None,
        load_from_rest_object_func: Optional[Callable] = None,
        nested_schema: Optional[Union[NestedField, List[NestedField]]] = None,
    ) -> None:
        """Register a type of node.

        :param _type: The type of the node.
        :type _type: str
        :keyword create_instance_func: A function to create a new instance of the node
        :paramtype create_instance_func: typing.Optional[typing.Callable[..., typing.Union[BaseNode, AutoMLJob]]]
        :keyword load_from_rest_object_func: A function to load a node from a rest object
        :paramtype load_from_rest_object_func: typing.Optional[typing.Callable[[Any], typing.Union[BaseNode, AutoMLJob\
            , ControlFlowNode]]]
        :keyword nested_schema: schema/schemas of corresponding nested field, will be used in \
            PipelineJobSchema.jobs.value
        :paramtype nested_schema: typing.Optional[typing.Union[NestedField, List[NestedField]]]
        """
        if create_instance_func is not None:
            self._create_instance_funcs[_type] = create_instance_func
        if load_from_rest_object_func is not None:
            self._load_from_rest_object_funcs[_type] = load_from_rest_object_func
        if nested_schema is not None:
            from azure.ai.ml._schema.core.fields import TypeSensitiveUnionField
            from azure.ai.ml._schema.pipeline.pipeline_component import PipelineComponentSchema
            from azure.ai.ml._schema.pipeline.pipeline_job import PipelineJobSchema

            for declared_fields in [
                PipelineJobSchema._declared_fields,
                PipelineComponentSchema._declared_fields,
            ]:
                jobs_value_field: TypeSensitiveUnionField = declared_fields["jobs"].value_field
                if not isinstance(nested_schema, list):
                    nested_schema = [nested_schema]
                for nested_field in nested_schema:
                    jobs_value_field.insert_type_sensitive_field(type_name=_type, field=nested_field)

    def load_from_dict(self, *, data: dict, _type: Optional[str] = None) -> Union[BaseNode, AutoMLJob]:
        """Load a node from a dict.

        :keyword data: A dict containing the node's data.
        :paramtype data: dict
        :keyword _type: The type of the node. If not specified, it will be inferred from the data.
        :paramtype _type: str
        :return: The node
        :rtype: Union[BaseNode, AutoMLJob]
        """
        if _type is None:
            _type = data[CommonYamlFields.TYPE] if CommonYamlFields.TYPE in data else NodeType.COMMAND
            # todo: refine Hard code for now to support different task type for DataTransfer node
            if _type == NodeType.DATA_TRANSFER:
                _type = "_".join([NodeType.DATA_TRANSFER, data.get("task", " ")])
        else:
            data[CommonYamlFields.TYPE] = _type

        new_instance: Union[BaseNode, AutoMLJob] = self.get_create_instance_func(_type)()

        if isinstance(new_instance, BaseNode):
            # parse component
            component_key = new_instance._get_component_attr_name()
            if component_key in data and isinstance(data[component_key], dict):
                data[component_key] = Component._load(
                    data=data[component_key],
                    yaml_path=data[component_key].pop(SOURCE_PATH_CONTEXT_KEY, None),
                )
        # TODO: Bug Item number: 2883415
        new_instance.__init__(**data)  # type: ignore
        return new_instance

    def load_from_rest_object(
        self, *, obj: dict, _type: Optional[str] = None, **kwargs: Any
    ) -> Union[BaseNode, AutoMLJob, ControlFlowNode]:
        """Load a node from a rest object.

        :keyword obj: A rest object containing the node's data.
        :paramtype obj: dict
        :keyword _type: The type of the node. If not specified, it will be inferred from the data.
        :paramtype _type: str
        :return: The node
        :rtype: Union[BaseNode, AutoMLJob, ControlFlowNode]
        """

        # TODO: Remove in PuP with native import job/component type support in MFE/Designer
        if "computeId" in obj and obj["computeId"] and obj["computeId"].endswith("/" + ComputeType.ADF):
            _type = NodeType.IMPORT

        if _type is None:
            _type = obj[CommonYamlFields.TYPE] if CommonYamlFields.TYPE in obj else NodeType.COMMAND
            # todo: refine Hard code for now to support different task type for DataTransfer node
            if _type == NodeType.DATA_TRANSFER:
                _type = "_".join([NodeType.DATA_TRANSFER, obj.get("task", " ")])
        else:
            obj[CommonYamlFields.TYPE] = _type

        res: Union[BaseNode, AutoMLJob, ControlFlowNode] = self.get_load_from_rest_object_func(_type)(obj, **kwargs)
        return res

    @classmethod
    def _automl_from_rest_object(cls, node: Dict) -> AutoMLJob:
        _outputs = cast(Dict[str, Union[str, dict]], node.get("outputs"))
        # rest dict outputs -> Output objects
        outputs = AutoMLJob._from_rest_outputs(_outputs)
        # Output objects -> yaml dict outputs
        parsed_outputs = {}
        for key, val in outputs.items():
            if isinstance(val, Output):
                val = val._to_dict()
            parsed_outputs[key] = val
        node["outputs"] = parsed_outputs
        return AutoMLJob._load_from_dict(
            node,
            context={BASE_PATH_CONTEXT_KEY: "./"},
            additional_message="Failed to load automl task from backend.",
            inside_pipeline=True,
        )


def _generate_component_function(
    component_entity: Component,
    override_definitions: Optional[Mapping[str, OverrideDefinition]] = None,  # pylint: disable=unused-argument
) -> Callable[..., Union[Command, Parallel]]:
    # Generate a function which returns a component node.
    def create_component_func(**kwargs: Any) -> Union[BaseNode, AutoMLJob]:
        # todo: refine Hard code for now to support different task type for DataTransfer node
        _type = component_entity.type
        if _type == NodeType.DATA_TRANSFER:
            # TODO: Bug Item number: 2883431
            _type = "_".join([NodeType.DATA_TRANSFER, component_entity.task])  # type: ignore
            if component_entity.task == DataTransferTaskType.IMPORT_DATA:  # type: ignore
                return pipeline_node_factory.load_from_dict(
                    data={"component": component_entity, "_from_component_func": True, **kwargs},
                    _type=_type,
                )
        return pipeline_node_factory.load_from_dict(
            data={"component": component_entity, "inputs": kwargs, "_from_component_func": True},
            _type=_type,
        )

    res: Callable = to_component_func(component_entity, create_component_func)
    return res


pipeline_node_factory = _PipelineNodeFactory()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=arguments-renamed

import logging
from typing import Optional

from azure.ai.ml._restclient.v2022_05_01.models import BatchRetrySettings as RestBatchRetrySettings
from azure.ai.ml._restclient.v2022_05_01.models import OnlineRequestSettings as RestOnlineRequestSettings
from azure.ai.ml._restclient.v2022_05_01.models import ProbeSettings as RestProbeSettings
from azure.ai.ml._utils.utils import (
    from_iso_duration_format,
    from_iso_duration_format_ms,
    to_iso_duration_format,
    to_iso_duration_format_ms,
)
from azure.ai.ml.entities._mixins import RestTranslatableMixin

module_logger = logging.getLogger(__name__)


class BatchRetrySettings(RestTranslatableMixin):
    """Retry settings for batch deployment.

    :param max_retries: Number of retries in failure, defaults to 3
    :type max_retries: int
    :param timeout: Timeout in seconds, defaults to 30
    :type timeout: int
    """

    def __init__(self, *, max_retries: Optional[int] = None, timeout: Optional[int] = None):
        self.max_retries = max_retries
        self.timeout = timeout

    def _to_rest_object(self) -> RestBatchRetrySettings:
        return RestBatchRetrySettings(
            max_retries=self.max_retries,
            timeout=to_iso_duration_format(self.timeout),
        )

    @classmethod
    def _from_rest_object(cls, settings: RestBatchRetrySettings) -> Optional["BatchRetrySettings"]:
        return (
            BatchRetrySettings(
                max_retries=settings.max_retries,
                timeout=from_iso_duration_format(settings.timeout),
            )
            if settings
            else None
        )

    def _merge_with(self, other: "BatchRetrySettings") -> None:
        if other:
            self.timeout = other.timeout or self.timeout
            self.max_retries = other.max_retries or self.max_retries


class OnlineRequestSettings(RestTranslatableMixin):
    """Request Settings entity.

    :param request_timeout_ms: defaults to 5000
    :type request_timeout_ms: int
    :param max_concurrent_requests_per_instance: defaults to 1
    :type max_concurrent_requests_per_instance: int
    :param max_queue_wait_ms: defaults to 500
    :type max_queue_wait_ms: int
    """

    def __init__(
        self,
        max_concurrent_requests_per_instance: Optional[int] = None,
        request_timeout_ms: Optional[int] = None,
        max_queue_wait_ms: Optional[int] = None,
    ):
        self.request_timeout_ms = request_timeout_ms
        self.max_concurrent_requests_per_instance = max_concurrent_requests_per_instance
        self.max_queue_wait_ms = max_queue_wait_ms

    def _to_rest_object(self) -> RestOnlineRequestSettings:
        return RestOnlineRequestSettings(
            max_queue_wait=to_iso_duration_format_ms(self.max_queue_wait_ms),
            max_concurrent_requests_per_instance=self.max_concurrent_requests_per_instance,
            request_timeout=to_iso_duration_format_ms(self.request_timeout_ms),
        )

    def _merge_with(self, other: Optional["OnlineRequestSettings"]) -> None:
        if other:
            self.max_concurrent_requests_per_instance = (
                other.max_concurrent_requests_per_instance or self.max_concurrent_requests_per_instance
            )
            self.request_timeout_ms = other.request_timeout_ms or self.request_timeout_ms
            self.max_queue_wait_ms = other.max_queue_wait_ms or self.max_queue_wait_ms

    @classmethod
    def _from_rest_object(cls, settings: RestOnlineRequestSettings) -> Optional["OnlineRequestSettings"]:
        return (
            OnlineRequestSettings(
                request_timeout_ms=from_iso_duration_format_ms(settings.request_timeout),
                max_concurrent_requests_per_instance=settings.max_concurrent_requests_per_instance,
                max_queue_wait_ms=from_iso_duration_format_ms(settings.max_queue_wait),
            )
            if settings
            else None
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, OnlineRequestSettings):
            return NotImplemented
        if not other:
            return False
        # only compare mutable fields
        return (
            self.max_concurrent_requests_per_instance == other.max_concurrent_requests_per_instance
            and self.request_timeout_ms == other.request_timeout_ms
            and self.max_queue_wait_ms == other.max_queue_wait_ms
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)


class ProbeSettings(RestTranslatableMixin):
    def __init__(
        self,
        *,
        failure_threshold: Optional[int] = None,
        success_threshold: Optional[int] = None,
        timeout: Optional[int] = None,
        period: Optional[int] = None,
        initial_delay: Optional[int] = None,
    ):
        """Settings on how to probe an endpoint.

        :param failure_threshold: Threshold for probe failures, defaults to 30
        :type failure_threshold: int
        :param success_threshold: Threshold for probe success, defaults to 1
        :type success_threshold: int
        :param timeout: timeout in seconds, defaults to 2
        :type timeout: int
        :param period: How often (in seconds) to perform the probe, defaults to 10
        :type period: int
        :param initial_delay: How long (in seconds) to wait for the first probe, defaults to 10
        :type initial_delay: int
        """

        self.failure_threshold = failure_threshold
        self.success_threshold = success_threshold
        self.timeout = timeout
        self.period = period
        self.initial_delay = initial_delay

    def _to_rest_object(self) -> RestProbeSettings:
        return RestProbeSettings(
            failure_threshold=self.failure_threshold,
            success_threshold=self.success_threshold,
            timeout=to_iso_duration_format(self.timeout),
            period=to_iso_duration_format(self.period),
            initial_delay=to_iso_duration_format(self.initial_delay),
        )

    def _merge_with(self, other: Optional["ProbeSettings"]) -> None:
        if other:
            self.failure_threshold = other.failure_threshold or self.failure_threshold
            self.success_threshold = other.success_threshold or self.success_threshold
            self.timeout = other.timeout or self.timeout
            self.period = other.period or self.period
            self.initial_delay = other.initial_delay or self.initial_delay

    @classmethod
    def _from_rest_object(cls, settings: RestProbeSettings) -> Optional["ProbeSettings"]:
        return (
            ProbeSettings(
                failure_threshold=settings.failure_threshold,
                success_threshold=settings.success_threshold,
                timeout=from_iso_duration_format(settings.timeout),
                period=from_iso_duration_format(settings.period),
                initial_delay=from_iso_duration_format(settings.initial_delay),
            )
            if settings
            else None
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ProbeSettings):
            return NotImplemented
        if not other:
            return False
        # only compare mutable fields
        return (
            self.failure_threshold == other.failure_threshold
            and self.success_threshold == other.success_threshold
            and self.timeout == other.timeout
            and self.period == other.period
            and self.initial_delay == other.initial_delay
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)
